# Add or Update Customer

    PUT /api/add-update-customer/{customerId?}

## Description
Returns customer object after storing/updating given customer information 

## Request Headers
- **X-Auth-Token** � Authorization Token
- **Content-Type** � application/json


## Parameter
- **CustomerId** � Customer Id is only required when a particular customer needs to be updated

## Return format
A JSON object containing key **Customer** 

- **Customer**  � Customer object


## Example
**Request**

    /api/add-update-customer

**Json Body with Contact Person & Proprietor Objects**
```javascript
{
	"area_id": 4,
	"biocos_ratting": 5.0,
	"category": "A+",
	"contact_person": {
		"address": "abc",
		"email": "",
		"landline": "",
		"name": "Zubair",
		"phone_1": "03215252369",
		"phone_2": ""
	},
	"currency_id": 2,
	"customer_type": "Wholesaler",
	"discount_percentage": 0.0,
	"product_discount": [],
	"email": "",
	"id": 0,
	"imagesList": [{
			"id": 0,
			"image_path": "/storage/emulated/0/Pictures/1527249918739.jpg",
			"images": ["/storage/emulated/0/Pictures/1527249918739.jpg"],
			"isSended": 0,
			"order_id": 0,
			"reference_id": 0,
			"shop_id": 0,
			"type": "Customer",
			"uuid": "ee59afd6-58bc-4d48-be2d-f71a4847db75"
		}
	],
	"job_id": 0,
	"latitude": 31.5144808,
	"location": "abc",
	"longitude": 74.3374753,
	"name": "Zubair",
	"opening_balance": 0.0,
	"stock": [],
	"phone_1": "03215252369",
	"phone_2": "",
	"proprietor": {
		"address": "abc",
		"email": "",
		"landline": "",
		"name": "Zubair",
		"phone_1": "03215252369",
		"phone_2": ""
	},
	"remove_images": [],
	"shop_type": "",
	"status": "Rejected",
	"uuid": "ee59afd6-58bc-4d48-be2d-f71a4847db75"
}

```

**Json Body with Contact Person and Proprietor ID**
```javascript
{
	"area_id": 1,
	"biocos_ratting": 5.0,
	"category": "A+",
	"contact_person": {
		"address": "Block E1, Lahore, Pakistan",
		"email": "",
		"landline": "",
		"name": "Jameel",
		"phone_1": "0321-6668448",
		"phone_2": ""
	},
	"customer_type": "Wholesaler",
	"discount_percentage": 3.0,
	"email": "",
	"images": [],
	"latitude": 31.51551750000001,
	"location": "Block E1, Lahore, Pakistan",
	"longitude": 74.33640234375001,
	"name": "Jameel",
	"phone_1": "0321-6668448",
	"phone_2": "",
	"proprietor": {
		"address": "Block E1, Lahore, Pakistan",
		"email": "",
		"landline": "",
		"name": "Jameel",
		"phone_1": "0321-6668448",
		"phone_2": ""
	},
	"remove_images": ["/storage/emulated/0/Pictures/JPEG_20180103_143214_-95396205.jpg", "/storage/emulated/0/Pictures/JPEG_20180103_143214_-95396205.jpg", "/storage/emulated/0/Pictures/JPEG_20180103_143201_-1701008546.jpg", "/storage/emulated/0/Pictures/JPEG_20180103_143208_-604929947.jpg"],
	"shop_type": "",
	"status": "Rejected",
	"uuid": "736edda0-7f3d-4376-ac00-fb245a7134d9"
}

```

**Return** __shortened for example purpose__
``` json
{
    "success": true,
    "message": "Customer Updated",
    "data": {
        "id": 4,
        "agency_id": null,
        "name": "Wood works",
        "proprietor_id": 1,
        "contact_person_id": 12,
        "location": "Liberty Market Lahore",
        "latitude": 31.577986,
        "longitude": 74.317994,
        "phone_1": "456456-56465-54",
        "phone_2": "55645-456645-44",
        "email": "admin@biocospk.com",
        "type": "wholesaler",
        "industry": "Super store",
        "discount_percentage": "retail saler",
        "biocos_ratting": 21.3,
        "region_id": 1,
        "status": "rejected",
        "Category": "D",
        "created_by": 1,
        "updated_by": 1,
        "deleted_by": 0,
        "created_at": "2017-07-09 20:58:14",
        "updated_at": "2017-07-09 20:59:13",
        "deleted_at": null
    }
}
```
 
