# Book or Update Order

    PUT /api/book-update-order/{orderId?}

## Description
Returns order object eager loading with products and quantities after storing/updating given order information 

## Request Headers
- **X-Auth-Token** — Authorization Token

## Parameter
- **OrderId** — Order Id is only required when a particular order needs to be updated

## Return format
A collection JSON object containing keys **Order** , **Products**

- **Order**  — Order object
- **Products**   —  Products and their corresponding quantities in order.


## Example
**Request**

    /api/book-update-order
**Json Body With Customer Object**
```javascript
{
	"customer": {
		"biocos_ratting": 0.0,
		"category": "",
		"contact_person": {
			"address": "",
			"email": "cu@admin.com",
			"landline": "",
			"name": "",
			"phone_1": "43434344",
			"phone_2": ""
		},
		"customer_type": "",
		"discount_percentage": 0.0,
		"email": "shop@test.com",
		"images": [],
		"latitude": 0.0,
		"location": "",
		"longitude": 0.0,
		"name": "Shop 1",
		"phone_1": "",
		"phone_2": "",
		"proprietor": {
			"address": "",
			"email": "cu@admin.com",
			"landline": "",
			"name": "",
			"phone_1": "43434344",
			"phone_2": ""
		},
		"region_id": 1,
		"shop_id": 1,
		"shop_type": "",
		"status": "",
		"uuid": "fc124db1-7627-40b5-a172-744b82203752"
	},
	"date_to_deliver": "2018-01-02",
	"delivery_time": "00:00",
	"discount": 0.0,
	"id": 0,
	"latitude": 31.5144147,
	"longitude": 74.3374838,
	"price": 7000.0,
	"products": [{
			"product_id": 1,
			"quantity": 50
		}
	],
	"remarks": "",
	"remove_images": [],
	"status": "Booked",
	"type": "Order",
	"uuid": "4696c0b4-f665-4614-a892-92b1dd52e3ba"
}

```

**Json Body**
```javascript
{
	"date_to_deliver": "2018-03-02",
	"delivery_time": "00:00",
	"discount": 2.0,
	"id": 0,
	"is_discounted": false,
	"is_discounted2": false,
	"job_id": 0,
	"latitude": 31.5144558,
	"longitude": 74.3375359,
	"payment": 7585.2,
	"price": 7740.0,
	"products": [{
			"product_actual_price": 2340.0,
			"product_discounted_price": 0.0,
			"scheme_free_quantity": 2,
			"product_id": 2,
			"quantity": 12,
			"product_category": []
		}, {
			"product_actual_price": 5400.0,
			"product_discounted_price": 0.0,
			"scheme_free_quantity": 0,
			"product_id": 12,
			"quantity": 12,
			"product_category": [{
					"agency_id": 0,
					"category": "",
					"created_at": "2018-03-01 11:09:35",
					"created_by": 0,
					"deleted_by": 0,
					"discount_id": 0,
					"distributor_price": 0.0,
					"id": 83,
					"is_scheme": 0,
					"minimum_order_unit": 0,
					"minimum_rs_quantity": 0,
					"minimum_ws_quantity": 0,
					"name": "Whitening Massage Cream Booster",
					"pivot": {
						"product_actual_price": 0.0,
						"product_discounted_price": 0.0,
						"scheme_free_quantity": 0,
						"order_id": 0,
						"product_id": 0,
						"quantity": 12
					},
					"product_id": 0,
					"quantity": 12,
					"retail_price": 0.0,
					"scheme_free_quantity": 0,
					"scheme_quantity": 0,
					"stock_available": 0,
					"product_category_id": 1,
					"type": "",
					"updated_at": "2018-03-01 11:09:35",
					"updated_by": 0,
					"wholesale_price": 0.0
				}
			]
		}
	],
	"remarks": "",
	"remove_images": [],
	"shop_id": 4,
	"shop_uuid": "58be9875-7ab7-45e7-a773-ef4188be6a2f",
	"status": "Booked",
	"type": "Order",
	"uuid": "c4ea4222-3bb7-4bcc-a98d-07641b449170",
	"visit_date": "2018-03-01"
}


```

**Return** __shortened for example purpose__
``` json
{
    "success": true,
    "message": "Order Updated",
    "data": {
        "id": 10,
        "agency_id": null,
        "customer_id": 1,
        "status": "booked",
        "visit_id": 0,
        "date_to_deliver": "2017-08-15 00:00:00",
        "booked_by": 1,
        "price": 25.2,
        "discount": 0,
        "type": "query",
        "remarks"		  : "Very urgent",
        "latitude"		  : "13.43",
        "longitude"       : "22.5",
        "delivery_time"   : "15:22",
        "discount"		  : "32.1",
        "created_by": 1,
        "updated_by": 1,
        "deleted_by": 0,
        "created_at": "2017-07-09 15:07:11",
        "updated_at": "2017-07-09 15:15:00",
        "deleted_at": null,
        "products": [
            {
                "id": 1,
                "agency_id": null,
                "name": "Cream",
                "product_code": "Cd-22",
                "category": "accessories",
                "type": "cream",
                "retail_price": 22.3,
                "wholesale_price": 20.3,
                "distributor_price": 25.33,
                "stock_available": 1,
                "started_on": "2017-07-01 18:25:06",
                "discontinued_on": "2018-07-06 18:52:47",
                "description": "asdasd",
                "minimum_order_unit": 2,
                "minimum_ws_quantity": 2,
                "minimum_rs_quantity": 2,
                "created_by": 1,
                "updated_by": 1,
                "deleted_by": 1,
                "created_at": null,
                "updated_at": null,
                "deleted_at": null,
                "pivot": {
                    "order_id": 10,
                    "product_id": 1,
                    "quantity": 3
                }
            },
            {
                "id": 2,
                "agency_id": null,
                "name": "Serum",
                "product_code": "Cd-23",
                "category": "accessories",
                "type": "cream",
                "retail_price": 22.3,
                "wholesale_price": 20.3,
                "distributor_price": 25.33,
                "stock_available": 1,
                "started_on": "2017-06-09 14:59:09",
                "discontinued_on": "2018-07-09 14:59:22",
                "description": "asdasd",
                "minimum_order_unit": 5,
                "minimum_ws_quantity": 5,
                "minimum_rs_quantity": 5,
                "created_by": 1,
                "updated_by": 1,
                "deleted_by": 1,
                "created_at": null,
                "updated_at": null,
                "deleted_at": null,
                "pivot": {
                    "order_id": 10,
                    "product_id": 2,
                    "quantity": 5
                }
            }
        ]
    }
}
```
 
