<?php

namespace App\Http\Controllers;

use App\Models\Customer;
use App\Models\Display;
use App\Models\Job;
use App\Models\JobDetail;
use App\Models\Order;
use App\Models\PaymentReceived;
use App\Models\PushNotificationToken;
use App\Models\User;
use App\Models\UserArea;
use App\Models\UserExpense;
use Dingo\Api\Facade\API;
use Illuminate\Http\Request;
use League\Flysystem\Exception;
use DB;
class CustomerController extends BaseController
{
    /***
     * @var Customer
     */
    private $_customerModel;

    /***
     * Constructor
     *
     * @param $request
     * @param Customer $customerModel
     */
    public function __construct(Request $request, Customer $customerModel)
    {
        set_time_limit(8000000);
        parent::__construct($request);
        $this->_customerModel = $customerModel;
    }

    /***
     * List customers
     *
     * @param $request
     * @return mixed
     */
    public function listCustomers(Request $request)
    {
        try {
            $userId = $request->get('user_id');
            $regionId = $request->get('region_id');
            $subRegion = $request->get('sub_region');
            $avoidPagination = $request->get('avoid_pagination', false);
            $page = $request->get('page', 1);
            if(IsNullOrEmptyString($userId)) {
                $userId = $this->getUserIdFromToken($request);
            }
            $customers = $this->_customerModel->getCustomers($userId, $regionId,
                                                             $subRegion, $avoidPagination, $page);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true, 'message' => 'Customers found',
            'data' => $customers], 200);
    }

    /***
     * Add or Update Customer
     *
     * @param Request $request
     * @param int $customerId
     * @return mixed
     */
    public function addOrUpdateCustomer(Request $request, $customerId = 0)
    {
        try {
//            $validator = \Validator::make($request->all(), $this->_customerModel->validationRules());

//            if ($validator->fails()) {
//                return API::response()->array(['success' => false, 'error' => 'Required parameters are missing or incorrect!',
//                                               'message' => $validator->errors()], 400);
//            } else {
                $customer = $request->all();
                $user = $this->getUserIdFromToken($request, true);
                $customer['user_id'] = $user->id;
                $customer['agency_id'] = $user->agency_id;
                $customer = $this->_customerModel->addOrUpdateCustomer($customer, $customerId,
                                                                       true);
                //Add Job for new Customer
//                if($customerId == 0) {
//                    $jobModel = new Job();
//                    $job = $jobModel->addJobForNewCustomer($customer);
//                    $customer->job = $job;
//                }
                if (is_null($customer)) {
                    return API::response()->array(['success' => false,
                                                   'error' => 'Customer Not Found'], 400);
                }
//            }
            return API::response()->array(['success' => true,
                'message' => 'Customer ' . (($customerId > 0) ? 'Updated' : 'Created'),
                'data' => $customer], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
    /***
     * Add or Update Customer Images
     *
     * @param Request $request
     * @return mixed
     */
    public function addOrUpdateImages(Request $request)
    {
        try {
            $data = $request->all();
            $type =  $data['type'];
            $customerId =  $data['id'];
            if($type == 'Customer'){
                $customerModel = new Customer();
                $customerId              = array_key_exists('id', $data)
                    ? $data['id'] : 0;
                if($customerId == 0) {
                    $customerId = $customerModel->getCustomerId($data);
                }
                if (($customerId == 0)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Customer Not Found'], 400);
                }
                $customerImage = $this->_customerModel->addOrUpdateCustomerImages($data,$customerId,
                    true);
                if (is_null($customerImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Customer Not Found'], 400);
                }
//                return $customerImage;
//                foreach ($customerImages as $customerImage){
                    $customerImage['type'] = $type;
                    $customerImage['uuid'] = $data['uuid'];
                    $customerImage['image_path'] = $data['image_path'];
                    $customerImage['reference_id'] = $customerImage['shop_id'];
//                }
                if(count($customerImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($customerImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $customerImage], 200);
                }
            }
            if($type == 'Display'){

                $customerImage = $this->_customerModel->addOrUpdateCustomerDisplayImages($data,
                    true);
                if (is_null($customerImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Customer Display Not Found'], 400);
                }
//                return $customerImage;
//                foreach ($customerImages as $customerImage){
                    $customerImage['type'] = $type;
                    $customerImage['uuid'] = $data['uuid'];
                    $customerImage['image_path'] = $data['image_path'];
//                    $customerImage['reference_id'] = $customerImage['shop_id'];

//                }
                if(count($customerImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($customerImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $customerImage], 200);
                }
            }
            if($type == 'Order'){
                $orderModel = new Order();
                $orderId              = array_key_exists('id', $data)
                    ? $data['id'] : 0;
                if($orderId == 0) {
                    $orderId = $orderModel->getOrderId($data);
                }
                if (($orderId == 0)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Order Not Found'], 400);
                }
                $orderImage = $orderModel->addOrUpdateOrderImage($data,$orderId,
                    true);
                if (is_null($orderImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Order Not Found'], 400);
                }
                $orderImage['type'] = $type;
                $orderImage['uuid'] = $data['uuid'];
                $orderImage['image_path'] = $data['image_path'];
                $orderImage['reference_id'] = $orderImage['order_id'];
                $orderImage['image_url'] = $orderImage['image'];
                if(count($orderImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($orderImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $orderImage], 200);
                }
            }
            if($type == 'No Order'){
                $jobDetailModel = new JobDetail();
                $jobDetailId              = array_key_exists('id', $data)
                    ? $data['id'] : 0;
                if($jobDetailId == 0) {
                    $jobDetailId = $jobDetailModel->getJobDetailId($data);
                }
                if (($jobDetailId == 0)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Job Not Found'], 400);
                }
                $jobDetailImage = $jobDetailModel->addOrUpdateJobImage($data,$jobDetailId,
                    true);
                if (is_null($jobDetailImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Job Not Found'], 400);
                }
//                return $jobDetailImage;
                $jobDetailImage['type'] = $type;
                $jobDetailImage['uuid'] = $data['uuid'];
                $jobDetailImage['image_path'] = $data['image_path'];
                $jobDetailImage['reference_id'] = $jobDetailImage['job_detail_id'];
//                $jobDetailImage['image_url'] = $orderImage['image_url'];
                if(count($jobDetailImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($jobDetailImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $jobDetailImage], 200);
                }
            }
            if($type == 'Payment'){
                $paymentModel = new PaymentReceived();
                $paymentId              = array_key_exists('id', $data)
                    ? $data['id'] : 0;
                if($paymentId == 0) {
                    $paymentId = $paymentModel->getPaymentId($data);
                }
                if (($paymentId == 0)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Payment Not Found'], 400);
                }
                $paymentImage = $paymentModel->addOrUpdatePaymentImage($data,$paymentId,
                    true);
                if (is_null($paymentImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Payment Not Found'], 400);
                }
                $paymentImage['type'] = $type;
                $paymentImage['uuid'] = $data['uuid'];
                $paymentImage['image_path'] = $data['image_path'];
                $paymentImage['reference_id'] = $paymentImage['payment_id'];
                $paymentImage['image_url'] = $paymentImage['image'];

                if(count($paymentImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($paymentImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $paymentImage], 200);
                }
            }
            if($type == 'Expense'){
                $userExpenseModel = new UserExpense();
                $userExpenseId              = array_key_exists('id', $data)
                    ? $data['id'] : 0;
                if($userExpenseId == 0) {
                    $userExpenseId = $userExpenseModel->getUserExpenseId($data);
                }
                if (($userExpenseId == 0)) {
                    return API::response()->array(['success' => false,
                        'error' => 'User Expense Not Found'], 400);
                }
                $userExpenseImage = $userExpenseModel->addOrUpdateUserExpenseImage($data,$userExpenseId,
                    true);
                if (is_null($userExpenseImage)) {
                    return API::response()->array(['success' => false,
                        'error' => 'User Expense Not Found'], 400);
                }
                $userExpenseImage['type'] = $type;
                $userExpenseImage['uuid'] = $data['uuid'];
                $userExpenseImage['image_url'] = $userExpenseImage['image'];

                if(count($userExpenseImage) > 0){
                    return API::response()->array(['success' => true,
                        'message' => $type . ((count($userExpenseImage) > 0) ? ' Image Updated' : ' Image Created'),
                        'data' => $userExpenseImage], 200);
                }
            }

                return API::response()->array(['success' => false,
                    'error' => 'No Images Created/Updated'], 400);



                //Add Job for new Customer
//                if($customerId == 0) {
//                    $jobModel = new Job();
//                    $job = $jobModel->addJobForNewCustomer($customer);
//                    $customer->job = $job;
//                }

        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }

    /***
 * Get Customer Details
 *
 * @param $customerId
 * @return mixed
 */
    public function getCustomerDetails(Request $request,$customerId)
    {
        try {
            $customer = $request->all();
            $user = $this->getUserIdFromToken($request, true);
            $customer['user_id'] = $user->id;
            $agency_id = $user->agency_id;
            $customer = $this->_customerModel->getCustomerDetail($customerId,$agency_id);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => is_null($customer) ? 'Customer not found' : 'Customer found',
            'data' => $customer], 200);
    }
    public function addDisplayimages(Request $request)
    {
        try {
            $customer = $request->all();
            $user = $this->getUserIdFromToken($request, true);
            $customer = $this->_customerModel->addOrUpdateCustomerDisplayWithoutImages($customer);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' =>  'Display Images added' ,
            'data' => $customer], 200);
    }
    public function addnotificationDetail(Request $request)
    {
        try {
            $customer = $request->all();
            $user = $this->getUserIdFromToken($request, true);
            PushNotificationToken::where('token',$request->get('token'))->where('user_id','!=',$request->get('user_id'))->delete();
            $customer = PushNotificationToken::where('user_id',$request->get('user_id'))
                ->where('model_number',$request->get('model_number'))
                ->first();
            if(empty($customer)){
                $customer = new PushNotificationToken();
            }
            $customer->user_id=$request->get('user_id');
            $customer->model_number=$request->get('model_number');
            $customer->token=$request->get('token');
            $customer->save();
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' =>  'Notification Detail added' ,
            'data' => $customer], 200);
    }
    public function getDisplayimageDetail(Request $request)
    {
        try {
            $user = $this->getUserIdFromToken($request, true);
            $display=Display::where('id',$request->get('id'))->with(['images','customer'])->first();
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' =>  'Display Images Details' ,
            'data' => $display], 200);
    }
    public function shopStatus(Request $request)
    {
        try {
            $job_id = $request->get('job_id');
            $job = Job::select('user_id','area_id')->where('id',$job_id)->first();
            if($job == ''){
                return API::response()->array(['success' => false,
                    'message' => 'Job not found'], 400);
            }
            $customers = Customer::select('id')
                ->where('sale_officer_id',$job->user_id)
                ->where('area_id',$job->area_id)
                ->get();
            if(count($customers) == 0){
                return API::response()->array(['success' => false,
                    'message' => 'Shop not found'], 400);
            }

            $job_detail = array();
            $i = 1;
            foreach ($customers as $customer){
                $result = JobDetail::select('shop_id','job_type')
                    ->where('shop_id',$customer->id)
                    ->where('job_id',$job_id)
//                    ->where('shop_id',4)
                    ->get();

                if(count($result) == 0){
                    $newdata =  array (
                        'shop_id' => $customer->id,
                        'job_type' => 'Pending'
                    );
                    $job_detail[] = $newdata;
                }else{
                    foreach ($result as $data){
                        $job_detail[] = $data;

                    }
                }
                $i++;

            }

        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => is_null($job_detail) ? 'Shop not found' : 'Shop found',
            'data' => $job_detail], 200);
    }
    public function listDistributer(Request $request)
    {
        try {
            $asm= UserArea::where('user_id',$request->get('asm_id'))->with(['area'=>function($query){return $query->with(['distributor'=>function($query){return $query->with('images');}]);}])->get();

//            $asm=User::select('id','firstname','lastname','image')
//                ->where('parent_id',$request->get('asm_id'))
//                ->orwhere('parent_id_2',$request->get('asm_id'))
//                ->get();
//            $user_ids=[];
//            $user_ids[]=$request->get('asm_id');
//            foreach ($asm as $cu){
//                $user_ids[]=$cu['id'];
//            }
//            $customers = Customer::where('customer_type','Distributer')
//                ->wherein('sale_officer_id',$user_ids)
//                ->with('images')
//                ->get();

        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => is_null($asm) ? 'Distributer not found' : 'Distributer found',
            'data' => $asm], 200);
    }
    public function shopDisplayRemarks(Request $request)
    {
        try {
            $shop_id = $request->get('shop_id');
            $remarks = $request->get('remarks');
            $customer = Customer::where('id',$shop_id)->first();
            if($customer == ''){
                return API::response()->array(['success' => false,
                    'message' => 'Shop not found'], 400);
            }
            $customer->update(array('display_remarks' => $remarks));

        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => 'Display Remarks Added'], 200);
    }
    /***
     * order filter by customer or region
     *
     * @param $customer_id or $region_id,start_date ,end_date
     * @return mixed
     */
    public function filterorder(Request $request)
    {
        $user_id=$this-> getUserIdFromToken($request,false);
        try {
            $order = $this->_customerModel->filteredOrder($request->all(),$user_id);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => count($order)>0 ? 'Order found' : 'Order not found',
            'data' => $order], 200);
    }
    public function filterallorder(Request $request)
    {
        $user_id=$this-> getUserIdFromToken($request,false);
        try {
            $order = $this->_customerModel->filteredAllOrder($request->all(),$user_id);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => count($order)>0 ? 'Order found' : 'Order not found',
            'data' => $order], 200);
    }
}