<?php

namespace App\Http\Controllers;

use App\Models\Job;
use App\Models\JobDetail;
use App\Models\JobDetailImage;
use Dingo\Api\Facade\API;
use Illuminate\Http\Request;
use League\Flysystem\Exception;

class JobController extends BaseController
{
    /***
     * @var Job
     */
    private $_jobModel;

    /***
     * Constructor
     *
     * @param $request
     * @param Job $jobModel
     */
    public function __construct(Request $request, Job $jobModel)
    {
        parent::__construct($request);
        $this->_jobModel = $jobModel;
    }

    /***
     *List jobs
     *
     * @param $request
     * @return mixed
     */
    public function listJobs(Request $request)
    {
        try {
            $userId = $request->get('user_id');
            $date = $request->get('date');
            $regionId = $request->get('region_id');
            $status = $request->get('status');
            $subRegion = $request->get('sub_region');
            $avoidPagination = $request->get('avoid_pagination', false);
            $page      = $request->get('page', 1);

            if(IsNullOrEmptyString($userId)) {
                $userId = $this->getUserIdFromToken($request);
            }
            $jobs = $this->_jobModel->getUserJobs($userId, $date, $regionId,
                                                  $status, $subRegion, $avoidPagination,
                                                  $page);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true, 'message' => 'Jobs found',
                                       'data' => $jobs], 200);
    }

    /***
     * Update jobs order
     *
     * @param Request $request
     * @return mixed
     */
    public function updateJobsOrder(Request $request)
    {
        try {
            $jobsToUpdate  = $request->all();
            if(count($jobsToUpdate) == 0) {
                return API::response()->array(['success' => false,
                    'error' => 'Required parameters are missing or incorrect!',
                    'message' => 'Data is missing!'], 400);
            }
            $userId = $this->getUserIdFromToken($request);
            $jobsUpdated = $this->_jobModel->updateJobOrder($jobsToUpdate, $userId);
            if($jobsUpdated) {
                return API::response()->array(['success' => true,
                    'message' => 'Jobs order Updated'], 200);
            } else {
                return API::response()->array(['success' => false,
                    'message' => 'Jobs not found or invalid request'], 400);
            }
        }
        catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }

    /***
     * Log No Order
     *
     * @param Request $request
     * @return mixed
     */
    public function logNoOrder(Request $request)
    {
        try {
            $job = $request->all();
            $userId = $this->getUserIdFromToken($request);

            $job_detail = JobDetail::where('job_id', $job['job_id'])
                ->where('shop_id', $job['shop_id'])
                ->first();
            if ($job_detail == '') {
                $jobDetail = new JobDetail();
                $job = $jobDetail->logNoOrderJob($job, $userId);
            }else{

            if ($job_detail->job_type == 'no-order') {
                return API::response()->array(['success' => true,
                    'message' => 'No Order job already added',
                    'data' => $job], 200);
            } elseif ($job_detail->job_type == 'revisit') {

                JobDetail::where('job_id', $job['job_id'])
                    ->where('shop_id', $job['shop_id'])
                    ->update(array('job_type' => 'no-order'));
                $job = JobDetail::where('job_id', $job['job_id'])->first();

            } elseif ($job_detail->job_type == 'payment') {
                $jobDetail = new JobDetail();
                $job = $jobDetail->logNoOrderJob($job, $userId);

            }
        }


//            $jobDetail= new JobDetail();
//            $job = $jobDetail->logNoOrderJob($job, $userId);
//            $job = JobDetailImage::logNoOrderJob($job, $userId);
//            $job = $this->_jobModel->logNoOrder($job, $userId);
            if(!is_null($job)) {
                return API::response()->array(['success' => true,
                    'message' => 'No order job added',
                    'data'   => $job], 200);
            } else {
                return API::response()->array(['success' => false,
                    'message' => 'invalid request'], 400);
            }
        }
        catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }

    /***
     * Log revisit
     *
     * @param Request $request
     * @return mixed
     */
    public function logRevisit(Request $request)
    {
        try {
            $job = $request->all();
            $userId = $this->getUserIdFromToken($request);
//            $job = $this->_jobModel->logRevisit($job, $userId);

            $job_detail = JobDetail::where('job_id', $job['job_id'])
                ->where('shop_id', $job['shop_id'])
                ->first();
            if ($job_detail == '') {
                $jobDetail = new JobDetail();
                $job = $jobDetail->logRevisit($job, $userId);
            }else{
            if ($job_detail->job_type == 'revisit') {
                return API::response()->array(['success' => true,
                    'message' => 'Revisit job already added',
                    'data' => $job], 200);
            } elseif ($job_detail->job_type == 'no-order') {

                JobDetail::where('job_id', $job['job_id'])
                    ->where('shop_id', $job['shop_id'])
                    ->update(array('job_type' => 'revisit'));
                $job = JobDetail::where('job_id', $job['job_id'])->first();

            }
        }

            if(!is_null($job)) {
                return API::response()->array(['success' => true,
                    'message' => 'Revisit job added',
                    'data'   => $job], 200);
            } else {
                return API::response()->array(['success' => false,
                    'message' => 'invalid request'], 400);
            }
        }
        catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
}