<?php

namespace App\Http\Controllers;

use App\Models\Area;
use App\Models\Job;
use App\Models\Region;
use App\Models\Role;
use App\Models\User;
use App\Models\PartyName;
use App\Models\UserRegion;
use Dingo\Api\Facade\API;
use Illuminate\Http\Request;
use League\Flysystem\Exception;

class RegionController extends BaseController
{
    /***
     * @var Region
     */
    private $_regionModel;

    /***
     * Constructor
     *
     * @param $request
     * @param Region $regionModel
     */
    public function __construct(Request $request, Region $regionModel)
    {
        parent::__construct($request);
        $this->_regionModel = $regionModel;
    }

    /***
     *List regions
     *
     * @param $request
     * @return mixed
     */
    public function listRegions(Request $request)
    {
        try {
            $userId = $request->get('user_id');
            $regionId = $request->get('region_id');
            $subRegion = $request->get('sub_region');
            $avoidPagination = $request->get('avoid_pagination', false);
            $page      = $request->get('page', 1);
            if(IsNullOrEmptyString($userId)) {
                $userId = $this->getUserIdFromToken($request);
            }
//            $regions = $this->_regionModel->getUserRegions($userId, $regionId, $subRegion,
//                                                           $avoidPagination , $page);
            $regions = $this->_regionModel->getUserRegions($userId, $regionId, $subRegion,
                                                           $avoidPagination , $page);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Regions found',
                                       'data' => $regions], 200);
    }
     public function listRegionsparty(Request $request)
    {
        try {
            $obj=Region::all();
            
        }
        
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Regions found',
                                       'data' => $obj], 200);
    }
    
      public function allRegions(Request $request,$userid=0)
    {
        try {
            $party_id='';
           if($userid==0){
             $userId = $this->getUserIdFromToken($request);
               
                $userrole = $this->getUserRole($userId);
             if($userrole == "gm"){
                $obj= Region::where('show_gm','=',1)->
               with(['partyname'])->get();
           }else{
        
                $obj= Region::
               with(['partyname'])->get();
           }
           }
          else if($userid==13)
           {
                $obj=Region::
                   with(['partyname'])->where('id',$userid)->get();
           }

           else{
               if($userid==103)
               {  
                    $party_id=showPartyToExport($userid);
                    $obj=Region::where('id',2)->with(['partyname'=>function($query) use($party_id){ 
                        
                        $query->wherein('id',$party_id);
                        
                        }])
                    
                    
                    ->get();
            
               }
               else{
                     $user=User::where('id',$userid)
               
               ->first();
               $obj=null;
               if(!empty($user)){
                   $obj=Region::where('id',$user->region_id)
                  ->with(['partyname'])->get();
               }
               }
               
             
           }
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Regions found',
                                       'data' => $obj], 200);
    }
     public function allexportRegions()
    {
        try {

               $obj= Region::with(['partyname' => function ($query) {
    $query->where('region_id',2)->where('id','!=',19);
}])->get();
              

        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Export Regions found',
                                       'data' => $obj], 200);
    }
    /***
     *List areas
     *
     * @param $request
     * @return mixed
     */
    public function listAreas(Request $request)
    {
        try {
            $userId = $request->get('user_id');
            $regionId = $request->get('area_id');
            $subRegion = $request->get('sub_region');
            $avoidPagination = $request->get('avoid_pagination', false);
            $page      = $request->get('page', 1);
            if(IsNullOrEmptyString($userId)) {
                $userId = $this->getUserIdFromToken($request);
            }
//            $regions = $this->_regionModel->getUserRegions($userId, $regionId, $subRegion,
//                                                           $avoidPagination , $page);
            $regions = $this->_regionModel->getUserAreas($userId, $regionId, $subRegion,
                                                           $avoidPagination , $page);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Areas found',
                                       'data' => $regions], 200);
    }
    public function roleDetails(Request $request)
    {
        try {
           $obj=Role::select('id','app_version','status_points')->where('id',$request->get('id'))->first();
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Role Detail',
                                       'data' => $obj], 200);
    }

    /***
     * Add or Update region
     *
     * @param Request $request
     * @param int $regionId
     * @return mixed
     */
    public function addOrUpdateRegion(Request $request, $regionId = 0)
    {
        try {
            $validator = \Validator::make($request->all(), $this->_regionModel->validationRules());

            if ($validator->fails()) {
                return API::response()->array(['success' => false, 'error' => 'Required parameters are missing or incorrect!',
                    'message' => $validator->errors()], 400);
            } else {
                $region = $request->all();
                $user = $this->getUserIdFromToken($request, true);
                $region['user_id'] = $user->id;
                $region['agency_id'] = $user->agency_id;
                $region = $this->_regionModel->addOrUpdateRegion($region, $regionId);
                if (is_null($region)) {
                    return API::response()->array(['success' => false,
                        'error' => 'Region Not Found'], 400);
                }
            }
            return API::response()->array(['success' => true,
                'message' => 'Region ' . (($regionId > 0) ? 'Updated' : 'Created'),
                'data' => $region], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }

    /***
     * Assign Or Update Regions
     *
     * @param Request $request
     * @return mixed
     */
    public function assignOrUpdateRegions(Request $request)
    {
        try {
            $addOrUpdateRegions = $request->get('update_region_model_list');
            $deleteUserRegions  = $request->get('delete_assign_region_model_list');
            $userModel = new User();
            $deletedUserRegions = [];
            $user = $this->getUserIdFromToken($request, true);
            //Delete User Regions
            if(!is_null($deleteUserRegions) && count($deleteUserRegions) > 0) {
                $userRegionModel = new UserRegion();
                foreach ($deleteUserRegions as $deleteUserRegion) {
                    if(array_key_exists('delete', $deleteUserRegion) && $deleteUserRegion['delete'] == "true") {
                        $deletedUserRegions[] = $userRegionModel->deleteUserRegion($deleteUserRegion['id']);
                    }
                }
            }
            //Assign or update regions
            $userRegions = $userModel->assignOrUpdateRegions($addOrUpdateRegions, $user->id);
            if (count($userRegions) == 0 && count($deletedUserRegions) == 0) {
                return API::response()->array(['success' => false,
                    'error' => 'Region Not Found'], 400);
            }
            return API::response()->array(['success' => true,
                'message' => 'User Regions Updated',
                'data' => ['user_regions' => $userRegions, 'deleted_regions' => $deletedUserRegions]], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
    /***
     * List user regions
     *
     * @param $request
     * @return mixed
     */
    public function listUserRegions(Request $request)
    {
        try {
            $userId = $this->getUserIdFromToken($request, false);
            $userModel = new User();
//            $regions = $userModel->getUserRegionsWithPivot($userId, [], true);
            $regions = $userModel->getUserAreasWithPivot($userId);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
            'message' => 'User Regions found',
            'data' => $regions], 200);
    }
    /***
     * List user areas
     *
     * @param $request
     * @return mixed
     */
    public function areasPerMonth(Request $request)
    {
        try {
        $user_id=$request->get('user_id');
        $originalDate = date('Y-m');
        $user_jobs=Job::where('user_id',$user_id)->get();
        $areasPerMonth =[];
        $date = new \DateTime($originalDate);
        $start=1;
        $end= date(' t ', strtotime($originalDate) );
        for($start;$start<=$end;$start++){
            $area_id=[];
            $b=0;
//            print_r($user_jobs);die();
            foreach ($user_jobs as $key=>$user_job ){  // 03-07-2018
                $diff=0;
                $date1=date_create($user_job['start_date']);  //03-07-2018
                $date2=date_create($user_job['start_date']);    //04-06-2018
                $diff=date_diff($date1,$date2);
                    for($a=0;$a<=$diff->format("%a");$a++){
                        if($date->format('Y-m-d')===$date1->format('Y-m-d')){
                            $area_id[$b]['area_id']=$user_job->area_id;
                            $area_id[$b]['job_id']=$user_job->id;
                            $b++;
                        }
                    $date1->modify('+1 day');
                }
            }
            $value=new \stdClass();
            $value->start_date=$date->format('Y-m-d') ;
            $value->areas=$area_id ;
            $areasPerMonth[]=$value;
            $date->modify('+1 day');
        }
    }
    catch(Exception $e)
    {
    return API::response()->array(['success' => false,
        'message' => $e->getTraceAsString()], 400);
    }
        return API::response()->array(['success' => true, 'message' => 'Areas Per Month',
            'data' => $areasPerMonth], 200);
    }
    /***
     * Update user areas
     *
     * @param $request
     * @return mixed
     */
    public function updatingAreas(Request $request)
    {
        try {
            $user_id=$request->get('user_id');
            $start_date=$request->get('start_date');
            $user_jobs=Job::where(['user_id'=>$user_id,'start_date'=>$start_date])->get();
            $areas=$request->get('areas');
            $area = [];
            if(count($user_jobs) > 0){
                foreach ($user_jobs as $user_job){
                    if(in_array($user_job->area_id,$areas)){
                    }else{
//                        return 'else';

                        Job::where(['user_id'=>$user_id,'area_id'=>$user_job->area_id,'start_date'=>$start_date])->delete();
                    }
                }
                if (!empty($areas)) {
                    $user_jobs=Job::where(['user_id'=>$user_id,'start_date'=>$start_date])->get()->toArray();

                    foreach ($areas as $key => $area_id) {
                        if (in_array($area_id, array_column($user_jobs, 'area_id'))) {
                            $area[$key]['area_id'] = $user_jobs[$key]['area_id'];
                            $area[$key]['job_id'] = $user_jobs[$key]['id'];
                        }else{
                            $job = new Job();
                            $user = User::where('id', $user_id)->first();
                            $job->area_id = $area_id;
                            $job->uuid = generate_uuid();
                            $job->user_id = $user_id;
                            $job->start_date = $start_date;
                            $job->agency_id = $user->agency_id;
                            $job->end_date = $start_date;
                            $job->status = 'Pending';
                            $job->comment = 'No Comment';
                            $job->save();
                            $area[$key]['area_id'] = $job->area_id;
                            $area[$key]['job_id'] = $job->id;
                        }
                    }
                }

            }else {
                if (!empty($areas)) {

                    foreach ($areas as $key => $area_id) {
                        $job = new Job();
                        $user = User::where('id', $user_id)->first();
                        $job->area_id = $area_id;
                        $job->uuid = generate_uuid();
                        $job->user_id = $user_id;
                        $job->start_date = $start_date;
                        $job->agency_id = $user->agency_id;
                        $job->end_date = $start_date;
                        $job->status = 'Pending';
                        $job->comment = 'No Comment';
                        $job->save();
                        $area[$key]['area_id'] = $job->area_id;
                        $area[$key]['job_id'] = $job->id;


                    }
                }
            }

            $value=new \stdClass();
            $value->start_date=$start_date ;
            $value->areas=$area ;
//            $value->areas=$areas ;
            $areasUpdated=$value;

    }
    catch(Exception $e)
    {
    return API::response()->array(['success' => false,
        'message' => $e->getTraceAsString()], 400);
    }

        return API::response()->array(['success' => true, 'message' => 'Successfully Updated',
            'data' => $areasUpdated], 200);
    }


}