<?php

namespace App\Http\Controllers;

use App\Models\UserLocation;
use Dingo\Api\Facade\API;
use Illuminate\Http\Request;
use League\Flysystem\Exception;
use App\Models\Product;
use App\Models\User;
use App\Models\PartyName;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Hash;
use DB;
class UserController extends BaseController
{
    /***
     * @var User
     */
    private $_userModel;

    /***
     * Constructor
     *
     * @param $request
     * @param User $userModel
     */
    public function __construct(Request $request, User $userModel)
    {
        parent::__construct($request);
        $this->_userModel = $userModel;
    }

    /***
     * Get user info
     *
     * @param int $userId
     * @param Request $request
     * @return mixed
     */
    public function userInfo($userId = 0, Request $request)
    {
        try {
            if($userId == 0) {
                $userId = $this->getUserIdFromToken($request);
            }
//            $userInfo = $this->_userModel->getUserInfoWithRegions(['id' => $userId ]);
            $userInfo = $this->_userModel->getUserInfoWithAreas(['id' => $userId ]);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => is_null($userInfo) ? 'User not found' : 'User found',
                                       'data' => $userInfo], 200);
    }
    
      public function userlists(Request $request)
    {
        try {
           
            $user = User::get();
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'User list successfully',
                                       'data' => $user], 200);
    }

 public function party_dead(Request $request)
    {
        try {
           
            $user = PartyName::where('id',$request->id)->first();
            if($user){
                $user->is_dead = $request->is_dead;
                $user->save();

            }
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                                           'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true,
                                       'message' => 'Party iś updated',
                                       'data' => $user], 200);
    }
    

    /***
     * Update profile
     *
     * @param Request $request
     * @param int $userId
     * @return mixed
     */
    public function updateProfile(Request $request, $userId = 0)
    {
        try {
            $validator = \Validator::make($request->all(), $this->_userModel->validationRulesUpdateProfile($userId));

            if ($validator->fails()) {
                return API::response()->array(['success' => false, 'error' => 'Required parameters are missing or incorrect!',
                    'message' => $validator->errors()], 400);
            } else {
                $user = $request->all();
                $user = $this->_userModel->updateUser($user, $userId);
                if (is_null($user)) {
                    return API::response()->array(['success' => false,
                        'error' => 'User Not Found'], 400);
                }
            }
            return API::response()->array(['success' => true,
                'message' => 'User Updated',
                'data' => $user], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
public function updateUserimage(Request $request)
    {
        try {
           
                $user = $request->all();
                $userId    = $request->id;

                $user = $this->_userModel->updateUserimage($user, $userId);
                if (is_null($user)) {
                    return API::response()->array(['success' => false,
                        'error' => 'User Not Found'], 400);
                }
            
            return API::response()->array(['success' => true,
                'message' => 'User image Updated',
                'data' => $user], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
    /***
     * Add user locations
     *
     * @param Request $request
     * @return mixed
     */
    public function addUserLocations(Request $request)
    {
        try {
            $userLocationModel = new UserLocation();
            $locations = $request->all();
            $userId = $this->getUserIdFromToken($request);
            foreach($locations as $location)
            {
                $location['user_id'] = $userId;
                $validator = \Validator::make($location, $userLocationModel->validationRules());
                if ($validator->fails()) {
                    return API::response()->array(['success' => false, 'error' => 'Required parameters are missing or incorrect!',
                        'message' => $validator->errors()], 400);
                } else {
                    $userLocationModel->insertUserLocation($location['user_id'], $location['latitude'],
                                                           $location['longitude'], $location['date']);
                }
            }
            return API::response()->array(['success' => true,
                'message' => 'Location updated'], 200);
        } catch(Exception $e) {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
    }
public function getasmuserregion(Request $request){

        try{
            $abc= [];
            $user=User::whereHas('roles', function($query) {
                $query->where('name', '=', 'asm');
            })
                ->where('region_id',$request->region_id)
                ->get();
            
           
           
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true, 'message' => 'asm users',
            'data' => $user], 200);


    }

   public function addvendor(Request $request)
    {

        try {
            $user='';


$done=User::where('username',$request->input('username'))->first();

      if(empty($done)) {

    $user = new User();


    if (!empty($request->input('password'))) {
        $user->password = Hash::make($request->input('password'));
    }

    $user->firstname = $request->input('firstname');
    $user->lastname = $request->input('lastname');
    $user->username = $request->input('username');
    $user->opening_balance = $request->input('opening_balance');
        $user->balance = $request->input('opening_balance');
    $user->agency_id = 1;
    $user->parent_id = 1;
    $user->country_id = 1;
    $user->region_id = 1;
    $user->city_id = 309;


    $user->joining_date = date('Y-m-d');
    $user->status = 1;

    $user=$user->save();
    $user=User::latest()->first();
    $done = DB::insert('insert into role_user (user_id, role_id) values (?, ?)', [$user->id, 10]);
}


        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }

        if(!empty($user))
        {
            return API::response()->array(['success' => true,'message'=>'user created sucuesfully',
                'data' => $user], 200);

        }


        else{
            return API::response()->array(['success' => false,'message'=>'user canot be created because '.$done->username.' already exist ',
                'data' => $user], 200);
        }


    }
    
      public function addparty(Request $request)
    {

        try {

                $party = new PartyName();
                $party->name = $request->input('name');
                $party->city = $request->input('city');
                $party->region_id = $request->input('region');
                $party->opening_balance = $request->input('opening_balance');
                $party->balance = $request->input('opening_balance');
                $party->discount=$request->input('discount');
$party->save();






        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }

       
        {
            return API::response()->array(['success' => true,'message'=>'Party created sucuesfully',
                'data' => $party], 200);

        }


       


    }
    
    /***
     * List user locations
     * 
     * @param Request $request
     * @return mixed
     */
    public function listUserLocations(Request $request)
    {
        try {
            $userLocationModel = new UserLocation();
            $userId = $request->get('user_id');
            $avoidPagination = $request->get('avoid_pagination', false);
            $page      = $request->get('page', 1);

            if(IsNullOrEmptyString($userId)) {
                $userId = $this->getUserIdFromToken($request);
            }
            $userLocations = $userLocationModel->getUserLocations($userId, $avoidPagination, $page);
        }
        catch(Exception $e)
        {
            return API::response()->array(['success' => false,
                'message' => $e->getTraceAsString()], 400);
        }
        return API::response()->array(['success' => true, 'message' => 'User Locations found',
            'data' => $userLocations], 200);
    }
    /**
    Test Mail.
     */
    public function Mail(Request $request)
    {
        $data=[];
        $data[]='a';
        Mail::send('email.test', $data, function($message){
            $message->from('adminpanel@biocospk.com', 'Biocos International Pvt Ltd');
            $message->to('shakir130@hotmail.com')->subject('Test');
        });
    }
}