<?php

namespace App\Models;

use Dingo\Api\Auth\Auth;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Validation\Rule;

class Customer extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */

    protected $table = 'shops';

    protected $guarded = ['id'];
//    public $appends = ['area_orders_count'];
//
//    public function getAreaOrdersCountAttribute()
//    {
//        return $this->customer_orders()->count() ;
//    }
    /***
     * Images of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function images()
    {
        return $this->hasMany('App\Models\CustomerImage', 'shop_id');
    }
    public function payments()
    {
        return $this->hasMany('App\Models\PaymentReceived', 'shop_id');
    }
//    public function customer_orders() {
//        return $this->hasMany('App\Models\Order','shop_id','id');
//
//    }
    /***
     * Stocks of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function stock()
    {
        return $this->hasMany('App\Models\ShopStock', 'shop_id');
    }
    /***
     * Products Discount of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function product_discount()
    {
        return $this->hasMany('App\Models\ShopProductDiscount', 'shop_id');
    }

    /***
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function proprietor()
    {
        return $this->belongsTo('App\Models\User', 'proprietor_id');
    }
    /***
     * Orders of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function orders()
    {
        return $this->hasMany('App\Models\Order', 'shop_id');
    }
    /***
     * Orders of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function ordersUuid()
    {
        return $this->hasMany('App\Models\Order', 'uuid');
    }

    /***
     * Contact Person Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function contactPerson()
    {
        return $this->belongsTo('App\Models\User', 'contact_person_id');
    }
    /***
     * Get customers based on filters
     *
     * @param $userId
     * @param $regionId
     * @param $subRegion
     * @param $avoidPagination
     * @param $page
     * @return mixed
     */
    public function getCustomers($userId, $regionId, $subRegion, $avoidPagination, $page = 1)
    {
        $regionIds = [];
        $regionModel = new Region();
        if(IsNullOrEmptyString($regionId)) {
//            $regionIds = $regionModel->getUserRegionIds($userId);
            $regionIds = $regionModel->getUserAreasIds($userId);
        } else {
            $regionIds [] = $regionId;
        }
        if($subRegion == 'true') {
            $regionIds = array_merge($regionIds, $regionModel->getSubRegions($regionIds, true));
        }
        $user = User::where('id',$userId)->first();
        $query = $this->with('images', 'proprietor', 'contactPerson','stock','product_discount')
                      ->where('agency_id', $user->agency_id)
                      ->whereIn('area_id', $regionIds);
        if($page > 0 && !$avoidPagination){
            $offset = calculate_offset($page);
            $query->skip($offset)
                ->take(10);
        }
        $shops = $query->get();
        return $shops;
    }

    /***
     * Validation Rules
     *
     * @param null $attributes
     * @return array
     */
    public function validationRules( $attributes = null )
    {
        $rules = [
            'uuid'            => 'required',
            'name'            => 'required',
            'location'        => 'required',
            'phone_1'         => 'required',
            'latitude'        => 'required|numeric',
            'longitude'       => 'required|numeric',
            'email'           => 'email',
            'customer_type'   => ['required', Rule::in(['Wholesaler','Retail Saler','Distributer'])],
            'shop_type'       => [Rule::in(['Parlor','Doctor','Medical Store','Pan Shop','Super Store',
                                                        'General Store','Cosmetics Shop',
                                                        'Tuk Shop at Fuel Station','Mobiler','Homeopathic Store',
                                                        'Pansar Store','Super Market'])],
            'discount_percentage' => ['required'],
            'status'          => ['required', Rule::in(['Approved','Pending','Rejected'])],
           
            'biocos_ratting'  => 'numeric',
            'region_id'       => 'numeric'
        ];

        // no list is provided
        if(!$attributes)
            return $rules;

        // a single attribute is provided
        if(!is_array($attributes))
            return [ $attributes => $rules[$attributes] ];

        // a list of attributes is provided
        $newRules = [];
        foreach ( $attributes as $attr )
            $newRules[$attr] = $rules[$attr];
        return $newRules;
    }

    /***
     * Add or update Customer
     *
     * @param $customer
     * @param int $customerId
     * @param bool $returnObj
     * @return mixed
     */
    public function addOrUpdateCustomer($customer, $customerId = 0, $returnObj = false)
    {

        $giveUserPoints = true;
        $customerObj = new $this();
        if($customerId > 0) {
            $customerObj = $this->where('id', $customerId)
                                ->first();
            $giveUserPoints = false;
            //Customer not found
            if(is_null($customerObj)) {
                return $customerObj;
            }
            $customerObj->updated_by = $customer['user_id'];
        } else {
            $customerObj->created_by = $customer['user_id'];
            if(array_key_exists('agency_id', $customer)) {
                $customerObj->agency_id = $customer['agency_id'];
            }
        }

        $customerObj->sale_officer_id        = $customer['user_id'];
        $customerObj->uuid         = $customer['uuid'];
        $customerObj->name         = $customer['name'];
        $customerObj->location     = $customer['location'];
        $customerObj->latitude     = $customer['latitude'];
        $customerObj->longitude    = $customer['longitude'];
        $customerObj->status       = $customer['status'];
        $customerObj->Category     = $customer['category'];
        $customerObj->area_id     = $customer['area_id'];
        $customerObj->currency_id     = $customer['currency_id'];
        $customerObj->opening_balance     = $customer['opening_balance'];
        $customerObj->balance     = (-1)* $customer['opening_balance'];
        $customerObj->landline     = $customer['landline'];

        $customerObj->discount_percentage = $customer['discount_percentage'];
        //Customer Type
        if(array_key_exists('customer_type', $customer)) {
            $customerObj->customer_type = $customer['customer_type'];
        }
        //Shop Type
        if(array_key_exists('shop_type', $customer)) {
            $customerObj->shop_type = $customer['shop_type'];
        }
        //Phone_1
        if(array_key_exists('phone_1', $customer)) {
            $customerObj->phone_1 = $customer['phone_1'];
        }
        //Proprietor ID
        if(array_key_exists('proprietor_id', $customer)) {
            $customerObj->proprietor_id = $customer['proprietor_id'];
        }
        elseif(array_key_exists('proprietor', $customer)) {
            $userModel = new User();
            $customerObj->proprietor_id = $userModel->createOrUpdateUser($customer['proprietor']);
        }
        //Contact Person ID
        if(array_key_exists('contact_person_id', $customer)) {
            $customerObj->contact_person_id = $customer['contact_person_id'];
        }
        elseif(array_key_exists('contact_person', $customer)) {
            $userModel = new User();
            $customerObj->contact_person_id = $userModel->createOrUpdateUser($customer['contact_person']);
        }
        //Phone 2
        if(array_key_exists('phone_2', $customer)) {
            $customerObj->phone_2 = $customer['phone_2'];
        }
        //Email
        if(array_key_exists('email', $customer)) {
            $customerObj->email = $customer['email'];
        }
        //Biocos Rating
        if(array_key_exists('biocos_ratting', $customer)) {
            $customerObj->biocos_ratting = $customer['biocos_ratting'];
        }
        //Region ID
//        if(array_key_exists('region_id', $customer)) {
//            $customerObj->region_id = $customer['region_id'];
//        }
        $customerObj->save();

        //Give user points
        if($giveUserPoints) {
            $userPointsModel = new UserPoint();
            //customer add points
            $userPointsModel->insertUserPoints($customer['user_id'], 1,$customerObj->id);
            //customer add target achievement points

            $add_tar_ac=SoTarget::where('so_id',$customer['user_id'])
                ->where('date', 'like', '%' . date("Y-m") . '%')
                ->first();
            $cus_count=$this->where('sale_officer_id',$customer['user_id'])
                ->where('created_at', 'like', '%' . date("Y-m-d") . '%')
                ->count();
            if($cus_count>=$add_tar_ac['customer_add'])
            $userPointsModel->insertUserPoints($customer['user_id'], 8,$customerObj->id);
        }
        //Customer Product Discount
        foreach ($customer['product_discount'] as $product_discount){

            $discount = ShopProductDiscount::where('shop_id',$customerObj->id)->
            where('product_id',$product_discount['product_id'])
                ->first();
            if($discount == ''){
                $shop_product_discount = new ShopProductDiscount();
                $shop_product_discount->shop_id = $customerObj->id;
                $shop_product_discount->product_id = $product_discount['product_id'];
                $shop_product_discount->product_discount_id = $product_discount['product_discount_id'];
                $shop_product_discount->save();

            }else{
                ShopProductDiscount::where('shop_id', $customerObj->id)
                    ->where('product_id', $product_discount['product_id'])
                    ->update(array('product_discount_id' => $product_discount['product_discount_id']));
            }
        }
        //Customer Stock
        foreach ($customer['stock'] as $stock){

            $customer_stock = ShopStock::where('shop_id',$customerObj->id)->
            where('product_id',$stock['product_id'])
                ->first();
            if($customer_stock == ''){
                $customerStock = new ShopStock();
                $customerStock->shop_id = $customerObj->id;
                $customerStock->product_id = $stock['product_id'];
                $customerStock->quantity = $stock['quantity'];
                $customerStock->save();

            }else{
                ShopStock::where('shop_id', $customerObj->id)
                    ->where('product_id', $stock['product_id'])
                    ->update(array('quantity' => $stock['quantity']));
            }
        }



        //Image upload
        if(array_key_exists('images', $customer)) {
            $images = [];
            foreach($customer['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/shops/',
                    'image-');
                $customerImage = new CustomerImage();
//                $customerImage->image_url =  str_replace("api.biocospk.com","",$image);
                $customerImage->image_url =  $image;
//                $customerImage->image_url =  'http://'.$image;
                $images[] = $customerImage;
            }
            if(count($images) > 0) {
                $customerObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $customer)) {
            $customerObj->images()->whereIn('image_url', $customer['remove_images'])
                        ->delete();
            foreach($customer['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/shops/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return ($returnObj) ? $this->getCustomerDetail($customerObj->id,$customerObj->agency_id) : $customerObj->id;
    }
    /***
     * Add or update Customer Images
     *
     * @param $customer
     * @param int $customerId
     * @param bool $returnObj
     * @return mixed
     */
    public function addOrUpdateCustomerImages($customer, $customerId = 0, $returnObj = false)
    {

        $customerObj = new $this();
        if($customerId > 0) {
            $customerObj = $this->where('id', $customerId)
                ->first();
            //Customer not found
            if(is_null($customerObj)) {
                return $customerObj;
            }
        }

        //Image upload
        if(array_key_exists('images', $customer)) {
            $images = [];
            foreach($customer['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/shops/',
                    'image-');
                $customerImage = new CustomerImage();
//                $customerImage->image_url =  str_replace("api.biocospk.com","",$image);
                $customerImage->image_url =  'http://'.$image;
                $customerImage->type =  'Other';
                $images[] = $customerImage;
            }
            if(count($images) > 0) {
                $customerObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $customer)) {
            $customerObj->images()->whereIn('image_url', $customer['remove_images'])
                        ->delete();
            foreach($customer['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/shops/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }

        return ($returnObj) ? $this->getCustomerImages($customerObj->id) : $customerObj->id;
    }
    /***
     * Add or update Customer Display Images
     *
     * @param $customer
     * @param int $customerId
     * @param bool $returnObj
     * @return mixed
     */
    public function addOrUpdateCustomerDisplayImages($customer, $returnObj = false)
    {

        $customerObj =  Display::where('uuid',$customer['uuid'])->first();
        if(is_null($customerObj)) {
            return $customerObj;
        }

        //Image upload
        if(array_key_exists('images', $customer)) {
            $images = [];
            foreach($customer['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/shops/',
                    'image-');
                $customerImage = new DisplayImages();
//                $customerImage->image_url =  str_replace("api.biocospk.com","",$image);
                $customerImage->image_url =  'http://'.$image;
                $images[] = $customerImage;
            }
            if(count($images) > 0) {
                $customerObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $customer)) {
            $customerObj->images()->whereIn('image_url', $customer['remove_images'])
                        ->delete();
            foreach($customer['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/shops/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return ($returnObj) ? $this->getCustomerDisplayImages($customerObj->id) : $customerObj->id;
    }
    public function addOrUpdateCustomerDisplayWithoutImages($displaydata){
        $display=new Display();
        $display->shop_id=$displaydata['shop_id'];
        $display->uuid=$displaydata['uuid'];
        $display->date=$displaydata['date'];
        $display->remarks=$displaydata['remarks'];
        $display->save();

        // Point added
        $point=new UserPoint();
        $point->user_id=$displaydata['so_id'];
        $point->point_type_id=7;
        $point->detail_id=$display->id;
        $point->date=$displaydata['date'];
        $point->save();

        return $display;
    }

    /***
     * Get Customer Detail
     *
     * @param $id
     * @return mixed
     */
    public function getCustomerDetail($id,$agency_id)
    {
        $customer = $this->with('images', 'contactPerson', 'proprietor','stock','product_discount')
                         ->where('id', $id)
                         ->where('agency_id', $agency_id)
                         ->first();
        return $customer;
    }
    /***
     * Get Customer Images
     *
     * @param $id
     * @return mixed
     */
    public function getCustomerImages($id)
    {
        $customer = CustomerImage::
                         where('shop_id', $id)
            ->orderBy('id', 'desc')
                         ->first();
        return $customer;
    }
    public function getCustomerDisplayImages($id)
    {
        $customer = DisplayImages::
                         where('display_id', $id)
            ->orderBy('id', 'desc')
                         ->first();
        return $customer;
    }

    /***
     * Get Customer ID using different params
     *
     * @param $customerArray
     * @return int
     */
    public function getCustomerId($customerArray)
    {
        $customerRecord =  $this->where('uuid', $customerArray['uuid'])
                                ->first();
        return is_null($customerRecord) ? 0 : $customerRecord->id;
    }
    /***
     * order filter using different params
     *
     * @param $customer_id or $region_id,start_date ,end_date
     * @return mixed
     */
    public function filteredOrder($data,$user_id)
    {

        $customer_id=[];
       if(isset($data['shop_id']) && $data['shop_id']!=0 ){

           $customer_id[]=$data['shop_id'];

       }elseif ($data['area_id'] == 0){

           $customers= $this::where('sale_officer_id',$data['so_id'])->select(['id'])->get()->toArray();

           foreach ($customers as $customer){
               $customer_id[]=$customer['id'];
           }
       }elseif (isset($data['area_id']) && $data['area_id']!=0){

           $customers= $this::where('area_id',$data['area_id'])->select(['id'])->get()->toArray();

           foreach ($customers as $customer){
               $customer_id[]=$customer['id'];
           }
       }
        $main_data = array();
        $order='';
        if(isset($customer_id) && count($customer_id)>0) {
           $main_data=[];
           if($data['type'] == 'order'){
               $order=Order::whereIn('shop_id',$customer_id)
                   ->where('booked_by',$user_id)
                   ->whereIn('type',['order','Cash','Credit'])
                   ->with(['images',
                       'products'=>function($query){
                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
                   ])->get();
           }
           elseif($data['type'] == 'no-order'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
//                   ->with(['images',
//                       'products'=>function($query){
//                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
//                   ])
                   ->get();
           }
           elseif($data['type'] == 'revisit'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
//                   ->where('booked_by',$user_id)->with(['images',
//                       'products'=>function($query){
//                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
//                   ])
                   ->get();
           }
           elseif($data['type'] == 'payment'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
//                   ->where('booked_by',$user_id)->with(['images',
//                       'products'=>function($query){
//                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
//                   ])
                   ->get();
           }
           elseif($data['type'] == 'Sample'){
               $order=Order::whereIn('shop_id',$customer_id)
                   ->where('booked_by',$user_id)
                   ->where('type',$data['type'])
                   ->with(['images',
                       'products'=>function($query){
                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
                   ])->get();
           }
           if(isset($data['start_date']) && !empty($data['start_date']) && isset($data['end_date']) && !empty($data['end_date']) ){
//               $order= $order->whereBetween('created_at', [$data['start_date'], $data['end_date']]);
//               $start_date = date_create($data['start_date']);
//               $start_date = $start_date->modify('-1 day');
               $end_date = date_create($data['end_date']);
//               $end_date = $end_date->modify('+1 day');
               $order= $order->where('created_at','>=', $data['start_date']);
               $order= $order->where('created_at','<=', $end_date->modify('+1 day'));
           }

//           $order= $order->with(['images',
//                   'products'=>function($query){
//                       $query->select('id','name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
//               ])->get()->toarray();
           foreach ($order as $value){
               if(isset($value['shop_id']) && $value['shop_id']!=''){
                   $customer_name=Customer::select('name','phone_1','location')->findorfail($value['shop_id'])->toarray();
                   $customer_image=CustomerImage::select('image_url')->where('shop_id',$value['shop_id'])->first();
                   $value['customer_name']=$customer_name['name'];
                   $value['phone']=$customer_name['phone_1'];
                   $value['location']=$customer_name['location'];
                   $value['customer_image']=$customer_image['image_url'];
               }
               $main_data[]=$value;
           }
       }
        return $main_data;
    }
    public function filteredAllOrder($data,$user_id)
    {

        $customer_id=[];
       if(isset($data['shop_id']) && $data['shop_id']!=0 ){

           $customer_id[]=$data['shop_id'];

       }elseif ($data['area_id'] == 0){

           $customers= $this::where('sale_officer_id',$data['so_id'])->select(['id'])->get()->toArray();

           foreach ($customers as $customer){
               $customer_id[]=$customer['id'];
           }
       }elseif (isset($data['area_id']) && $data['area_id']!=0){

           $customers= $this::where('area_id',$data['area_id'])->select(['id'])->get()->toArray();

           foreach ($customers as $customer){
               $customer_id[]=$customer['id'];
           }
       }
        $main_data = array();
        $order='';
        if(isset($customer_id) && count($customer_id)>0) {
           $main_data=[];
           if($data['type'] == 'order'){
               $order=Order::whereIn('shop_id',$customer_id)
                   ->whereIn('type',['order','Cash','Credit'])
                   ->with(['images',
                       'products'=>function($query){
                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
                   ])->get();
           }
           elseif($data['type'] == 'no-order'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
//                   ->with(['images',
//                       'products'=>function($query){
//                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
//                   ])
                   ->get();
           }
           elseif($data['type'] == 'revisit'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
                   ->get();
           }
           elseif($data['type'] == 'payment'){
               $order=JobDetail::whereIn('shop_id',$customer_id)
                   ->where('job_type',$data['type'])
                   ->get();
           }
           elseif($data['type'] == 'Sample'){
               $order=Order::whereIn('shop_id',$customer_id)
                   ->where('type',$data['type'])
                   ->with(['images',
                       'products'=>function($query){
                           $query->select('name','product_code','retail_price','wholesale_price','distributor_price')->with('images');}
                   ])->get();
           }
           if(isset($data['start_date']) && !empty($data['start_date']) && isset($data['end_date']) && !empty($data['end_date']) ){
               $end_date = date_create($data['end_date']);
//               $order= $order ->whereBetween('date_to_deliver', [$data['start_date'],$data['end_date']]);

               $order= $order->where('date_to_deliver','>=', $data['start_date']);
               $order= $order->where('date_to_deliver','<=', $data['end_date']);
//               $order= $order->where('created_at','>=', $data['start_date']);
//               $order= $order->where('created_at','<=', $end_date->modify('+1 day'));
           }
           foreach ($order as $value){
               if(isset($value['shop_id']) && $value['shop_id']!=''){
                   $customer_name=Customer::select('name','phone_1','location')->findorfail($value['shop_id'])->toarray();
                   $customer_image=CustomerImage::select('image_url')->where('shop_id',$value['shop_id'])->first();
                   $value['customer_name']=$customer_name['name'];
                   $value['phone']=$customer_name['phone_1'];
                   $value['location']=$customer_name['location'];
                   $value['customer_image']=$customer_image['image_url'];
               }
               $main_data[]=$value;
           }
       }
        return $main_data;
    }
}