<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class JobDetail extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */

    protected $table = 'job_details';

    protected $guarded = ['id'];
    /***
     * Images of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function images()
    {
        return $this->hasMany('App\Models\JobDetailImage', 'job_detail_id','id');
    }
    /***
     * Get Job Detail ID using uuid
     *
     * @param $uuid
     * @return int
     */
    public function getJobDetailId($uuid)
    {
        $jobDetailRecord =  $this->where('uuid', $uuid['uuid'])
            ->first();
        return is_null($jobDetailRecord) ? 0 : $jobDetailRecord->id;
    }
    public function logNoOrderJob($job, $userId)
    {
        //add job detail
        $job_detail = new JobDetail();
        $job_detail->uuid = $job['uuid'];
        $job_detail->job_id = $job['job_id'];
        $job_detail->shop_id = $job['shop_id'];
        $job_detail->visit_date = $job['visit_date'];
        $job_detail->job_type = 'no-order';
        $job_detail->comment = $job['remarks'];
        $job_detail->reason = $job['reason'];
        $job_detail->latitude = $job['latitude'];
        $job_detail->longitude = $job['longitude'];
        $job_detail->save();

        //Image upload
        if(array_key_exists('images', $job)) {
            $images = [];
            foreach($job['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/jobs/',
                    'image-');
                $JobDetailImage = new JobDetailImage();
                $JobDetailImage->image_url = 'http://'.$image;
                $images[] = $JobDetailImage;
            }
            if(count($images) > 0) {
                $job_detail->images()->saveMany($images);
            }
        }
        return $this->where('id', $job_detail->id)->first();
    }
    public function logRevisit($job, $userId)
    {
        //add job detail
        $job_detail = new JobDetail();
        $job_detail->uuid = $job['uuid'];
        $job_detail->job_id = $job['job_id'];
        $job_detail->shop_id = $job['shop_id'];
        $job_detail->visit_date = $job['visit_date'];
        $job_detail->job_type = 'revisit';
        $job_detail->comment = '';
        $job_detail->reason = '';
        $job_detail->latitude = $job['latitude'];
        $job_detail->longitude = $job['longitude'];
        $job_detail->revisit_on = $job['revisit_on'];
        $job_detail->save();
        //Image upload
        if(array_key_exists('images', $job)) {
            $images = [];
            foreach($job['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/jobs/',
                    'image-');
                $JobDetailImage = new JobDetailImage();
                $JobDetailImage->image_url = 'http://'.$image;
                $images[] = $JobDetailImage;
            }
            if(count($images) > 0) {
                $job_detail->images()->saveMany($images);
            }
        }
        return $this->where('id', $job_detail->id)->first();
    }
    /***
     * Add or Update Job Image
     *
     * @param $jobDetail
     * @param int $jobDetailId
     *
     * @return mixed
     */
    public function addOrUpdateJobImage($jobDetail, $jobDetailId = 0)
    {

        $jobDetailObj = new $this();
        if ($jobDetailId > 0) {
            $jobDetailObj = $this->where('id', $jobDetailId)
                ->first();

            //Order not found
            if (is_null($jobDetailObj)) {
                return $jobDetailObj;
            }
        }
        //Image upload
        if(array_key_exists('images', $jobDetail)) {
            $images = [];
            foreach($jobDetail['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/jobs/',
                    'image-');
                $JobDetailImage = new JobDetailImage();
                $JobDetailImage->image_url = 'http://'.$image;
                $images[] = $JobDetailImage;
            }
            if(count($images) > 0) {
                $jobDetailObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $jobDetail)) {
            $jobDetailObj->images()->whereIn('image', $jobDetail['remove_images'])
                ->delete();
            foreach($jobDetail['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/jobs/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return $this->getJobImage($jobDetailObj->id);
    }
    /***
     * Get Order Images
     *
     * @param $id
     * @return mixed
     */
    public function getJobImage($id)
    {
        $order = JobDetailImage::
        where('job_detail_id', $id)
            ->orderBy('id', 'desc')
            ->first();
        return $order;
    }

}
