<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Mail;
use Auth;
use DB;

class Order extends Model
{
    use SoftDeletes;
    /***
     * Table name
     *
     * @var string
     */
    protected $table = 'orders';

    protected $appends = ['is_discounted_2'];
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = ['id'];
    /***
     * Images of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function images()
    {
        return $this->hasMany('App\Models\OrderImage', 'order_id');
    }
    public function getisdiscounted2attribute()
    {
        $val= $this->is_discounted;
        if($val==1){
            return true;
        }else{
            return false;
        }


    }
    /***
     * Validation rules for add user
     *
     * @param null $attributes
     * @return array
     */
    public function validationRules( $attributes = null )
    {
        $rules = [
            'uuid'            => 'required',
            'customer_id'     => 'integer',
            'status'          => [ 'required', Rule::in(['Booked','Confirmed','Processed','Ready',
                                                         'Delivered','Cleared','Rejected','Cancelled','Returned']) ],
            'date_to_deliver' => 'required|date',
            'price'           => 'required|numeric',
            'discount'        => 'numeric',
            'products'        => 'required',
            'type'            => [ 'required', Rule::in([ 'Query','Order','Sample','Cash','Credit' ]) ]
        ];

        // no list is provided
        if(!$attributes)
            return $rules;

        // a single attribute is provided
        if(!is_array($attributes))
            return [ $attributes => $rules[$attributes] ];

        // a list of attributes is provided
        $newRules = [];
        foreach ( $attributes as $attr )
            $newRules[$attr] = $rules[$attr];
        return $newRules;
    }
    /***
     * Relationship with products
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function products()
    {
        return $this->belongsToMany('App\Models\Product', 'order_products',
            'order_id', 'product_id')
            ->withPivot('id','quantity','product_actual_price','product_discounted_price','scheme_free_quantity');
    }
    public function area_orders() {
        return $this->belongsToMany('App\Models\Order', 'shops',
            'sale_officer_id', 'area_id');
    }
    public function book_by()
    {
        return $this->belongsTo('App\Models\User','booked_by');
    }

    /***
     * Customer Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function customer()
    {
        return $this->belongsTo('App\Models\Customer', 'shop_id');
    }
    public function order_products()
    {
        return $this->hasMany('App\Models\OrderProduct', 'order_id');
    }
    /***
     * CustomerUuid Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function customerUuid()
    {
        return $this->belongsTo('App\Models\Customer', 'uuid');
    }

    /***
     * Payment Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function payments()
    {
        return $this->hasMany('App\Models\PaymentReceived', 'order_id');
    }

    /***
     * Add or Update Order
     *
     * @param $order
     * @param int $orderId
     *
     * @return mixed
     */
    public function addOrUpdateOrder($order,$orderId = 0)
    {
        $giveUserPoints = true;
        $userPointsModel = new UserPoint();
        $orderObj = new $this();
        if ($orderId > 0) {
            $orderObj = $this->with('products')
                ->where('id', $orderId)
                ->first();
            $giveUserPoints = false;
            //Order not found
            if (is_null($orderObj)) {
                return $orderObj;
            }
            $orderObj->updated_by = $order['user_id'];
        } else {
            $o_count=$this->where('shop_id', $order['shop_id'])->count();
            if($o_count>0)
                $giveUserPoints = false;
            $user_country_id = User::select('country_id')->where('id',$order['user_id'])->first();
            $orderObj->country_id = $user_country_id->country_id;
            $orderObj->booked_by = $order['user_id'];
            $orderObj->created_by = $order['user_id'];
            $orderObj->agency_id = $order['agency_id'];
        }
        // getting RSM,ASM or ASE of Order Booker
//        $distributer_user_id=0;
//        $rsm_id=User::where('id',$order['user_id'])->first();
//        if(!empty($rsm_id) && $rsm_id['parent_type']=='rsm' && !empty($rsm_id['rsm']) && $rsm_id['rsm']!=0){
//            $distributer_user_id=$rsm_id['rsm'];
//        }else{
//            $asm=User::where('agency_id',Auth::user()->agency_id)->whereHas('roles', function($query) {
//                $query->where('name', '=', 'asm');
//            })->get();
//        }
        
        //Contact Person ID

        if (array_key_exists('shop_id', $order)) {
            if($order['shop_id'] != 0){
                $orderObj->shop_id = $order['shop_id'];
            }else{
                $customerModel = new Customer();
                $customerId = $customerModel->getCustomerId($order['customer']);
//                $customerId = Customer::where('uuid',$order['customer']['uuid'])->first();
                $orderObj->shop_id = $customerId;
            }
        } elseif (array_key_exists('customer', $order)) {

            $customerModel = new Customer();
            $customerId = $customerModel->getCustomerId($order['customer']);
            $orderObj->shop_id = $customerId;
//            $order['customer']['user_id'] = $order['user_id'];
//            $order['shop_id'] = $customerModel->addOrUpdateCustomer($order['customer'],
//                $customerId);
        }
//        $orderObj->shop_id = $order['shop_id'];
        $orderObj->status = $order['status'];
        $orderObj->date_to_deliver = $order['date_to_deliver'];
        $orderObj->price = round($order['price'],2);
        $orderObj->uuid = $order['uuid'];
        $orderObj->is_discounted = $order['is_discounted'];
//        if($order['is_discounted'] == true){
//            $orderObj->is_discounted = 1;
//        }else{
//            $orderObj->is_discounted = 0;
//        }


        if (array_key_exists('remarks', $order)) {
            $orderObj->remarks = $order['remarks'];
        }

        if (array_key_exists('payment', $order)) {
            $orderObj->payment = round($order['payment'],2);
        }

        if (array_key_exists('delivery_time', $order)) {
            $orderObj->delivery_time = $order['delivery_time'];
        }

        if (array_key_exists('remarks', $order)) {
            $orderObj->remarks = $order['remarks'];
        }

        if (array_key_exists('latitude', $order)) {
            $orderObj->latitude = $order['latitude'];
        }

        if (array_key_exists('longitude', $order)) {
            $orderObj->longitude = $order['longitude'];
        }

        if (array_key_exists('discount', $order)) {
            $orderObj->discount = $order['discount'];
        }
        $orderObj->type = $order['type'];
        $orderObj->save();
        $first=date('Y-m-d', strtotime('-30 days'));
//        $first=date('Y-m-01',strtotime('last month'));
        $last=date('Y-m-d');
//        $last=date('Y-m-t',strtotime('last month'));
        $rep_count=$this->where('shop_id', $order['shop_id'])
            ->whereBetween('date_to_deliver', [$first, $last])
            ->count();
//        $cur_m_order=Order::where('shop_id', $order['shop_id'])
//            ->whereBetween('date_to_deliver', [date('Y-m-01'), date('Y-m-t')])
//            ->where('booked_by', $order['user_id'])
//            ->count();
        $productive_shop=$this->where('booked_by', $order['user_id'])
            ->Where('created_at', 'like', '%' . date("Y-m-d") . '%')
//            ->where('date_to_deliver', date("Y-m-d"))
            ->count();
        $Role = User::find( $order['user_id'] )->roles->first()->name ;
        if($Role=='so'){
            $target=TargetDetail::where('date', 'like', '%' . date("Y-m") . '%')->first();
            if(!empty($target)){
                $sales=$this->where('booked_by', $order['user_id'])
                    ->where('date_to_deliver', 'like', '%' . date("Y-m") . '%')
                    ->where('status','=','Delivered')->where('type','!=','Sample')
                    ->sum('payment');
                if($sales>=$target['sales_target']){
                    $userPointsModel->insertUserPoints($order['user_id'], 3,$orderObj->id);
                }
            }
        }
//        $sales_target=

        if($productive_shop<25){
            $userPointsModel->insertUserPoints($order['user_id'], 5,$orderObj->id);
        }elseif($productive_shop==25){
            $userPointsModel->insertUserPoints($order['user_id'], 10,$orderObj->id);
        }elseif($productive_shop>25){
            $userPointsModel->insertUserPoints($order['user_id'], 11,$orderObj->id);
        }
//        if($rep_count>0 && $cur_m_order<1){
        if($rep_count>0 ){
            $userPointsModel->insertUserPoints($order['user_id'], 6,$orderObj->id);
        }
        //Give user points
        if ($giveUserPoints) {
            $userPointsModel->insertUserPoints($order['user_id'], 2,$orderObj->id);
        }
        //update job status
        $job = Job::where('id', $order['job_id'])
            ->first();
        if($job != ""){
            $job = Job::where('id', $order['job_id'])
                ->update(array('status' => 'Completed','end_date' => $orderObj->created_at));
        }
        //add job detail
        $job_details = JobDetail::where('job_id', $order['job_id'])
            ->where('shop_id', $order['shop_id'])
            ->get();
        if(count($job_details)== 0){
            $job_detail = new JobDetail();
            $job_detail->uuid = $order['uuid'];
            $job_detail->job_id = $order['job_id'];
            $job_detail->shop_id = $orderObj->shop_id;
            $job_detail->visit_date = $order['visit_date'];
            $job_detail->job_type = 'order';
            $job_detail->job_type_id = $orderObj->id;
            $job_detail->comment = $order['remarks'];
            $job_detail->reason = '';
            $job_detail->save();
            //Image upload
            if (array_key_exists('images', $order)) {
                $images = [];
                foreach ($order['images'] as $image) {
                    $image = upload_base64_image($image, 'uploads/order/',
                        'image-');
                    $orderImage = new OrderImage();
                    $orderImage->image = 'http://' . $image;
                    $images[] = $orderImage;
                }
                if (count($images) > 0) {
                    $orderObj->images()->saveMany($images);
                }
            }
        }else {
            $already = 0;
            $already_added = 0;
            foreach ($job_details as $job_detail) {
                    if ($job_detail->job_type == 'no-order') {
                        if($already_added == 1){
                            $job = JobDetail::where('job_id', $order['job_id'])
                                ->where('shop_id', $order['shop_id'])
                                ->where('job_type', 'no-order')
                                ->delete();
                        }else{
                            $job = JobDetail::where('job_id', $order['job_id'])
                                ->where('shop_id', $order['shop_id'])
                                ->where('job_type', 'no-order')
                                ->update(array('job_type' => 'order'));
                        }

                        $already = 1;

                    } elseif ($job_detail->job_type == 'revisit') {
                        JobDetail::where('job_id', $order['job_id'])
                            ->where('shop_id', $order['shop_id'])
                            ->where('job_type', 'revisit')
                            ->update(array('job_type' => 'order'));
                    } elseif ($job_detail->job_type == 'payment') {
                        if ($already == 0) {
                            $already_added = 1;
                            $job_detail = new JobDetail();
                            $job_detail->uuid = $order['uuid'];
                            $job_detail->job_id = $order['job_id'];
                            $job_detail->shop_id = $order['shop_id'];
                            $job_detail->visit_date = $order['visit_date'];
                            $job_detail->job_type = 'order';
                            $job_detail->job_type_id = $orderObj->id;
                            $job_detail->comment = $order['remarks'];
                            $job_detail->reason = '';
                            $job_detail->save();
                            //Image upload
                            if (array_key_exists('images', $order)) {
                                $images = [];
                                foreach ($order['images'] as $image) {
                                    $image = upload_base64_image($image, 'uploads/order/',
                                        'image-');
                                    $orderImage = new OrderImage();
                                    $orderImage->image = 'http://' . $image;
                                    $images[] = $orderImage;
                                }
                                if (count($images) > 0) {
                                    $orderObj->images()->saveMany($images);
                                }
                            }
                        }
                    }
                }
            }


        //Image upload
//        if (array_key_exists('images', $order)) {
//            $images = [];
//            foreach ($order['images'] as $image) {
//                $image = upload_base64_image($image, 'uploads/order/',
//                    'image-');
//                $orderImage = new OrderImage();
//                $orderImage->image = 'http://' . $image;
//                $images[] = $orderImage;
//            }
//            if (count($images) > 0) {
//                $orderObj->images()->saveMany($images);
//            }
//        }


        //Remove Image
        if(array_key_exists('remove_images', $order)) {
            $orderObj->images()->whereIn('image', $order['remove_images'])
                ->delete();
            foreach($order['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/order/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }

        if(array_key_exists('products', $order)) {
            $products = [];
            $Order_Products=[];
            $check_values=[];
            $Order_Products_category=[];
            $check_values_category=[];
            $OrderProduct2 = OrderProduct::where('order_id',$orderObj->id)
                ->get();
            foreach ($OrderProduct2 as $value){
                $Order_Products[]= $value['product_id'];
            }
            foreach($order['products'] as $key =>$product) {

                //update distributer stock
                $customer=Customer::where('id',$order['shop_id'])->first();
                if(!empty($customer)){
                    $dis_id=Area::where('id',$customer->area_id)->first();
                    if(!empty($dis_id)){
                        $distributer_stock=DistributerStock::where('asm_id',$order['user_id'])
                            -> where('shop_id',$dis_id->shop_id)
                            -> where('date',date('Y-m-d'))
                            ->first();
                        if(!empty($distributer_stock)){
                            $dis_product=DistributerStockDetail::where('distributer_stock_id',$distributer_stock['id'])
                                ->where('product_id',$product['product_id'])
                                ->first();
                            if(!empty($dis_product)){
                                if($orderId==0){
                                    $dis_product->remaining=$dis_product->remaining-$product['quantity'];
                                    $dis_product->booked=$dis_product->booked+$product['quantity'];
                                    $dis_product->save();
                                }else{

                                    $new_remaining = $dis_product->booked - $product['quantity'];
                                    $new_book = $dis_product->booked - $product['quantity'];
                                    if($new_book<0){
                                        $new_book=-1*$new_book;
                                    }
                                    if($new_remaining<0){
                                        $new_book=-1*$new_remaining;
                                    }
                                    $dis_product->remaining=$new_remaining;
                                    $dis_product->booked=$new_book;
                                    $dis_product->save();
                                }
                            }
                        }
                    }

                }


                $check_values[] = $product['product_id'];
                $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                    ->where('product_id',$product['product_id'])
                    ->first();
                if($order['type'] == 'Sample'){
                    $so_stock = SoStock::where('so_id',$order['user_id'])->where('product_id',$product['product_id'])->first();
                    if($so_stock != '') {
                        $old_stock = $so_stock['qty'];
                        $new_stock = $product['quantity'];
                        $total_stock = $old_stock - $new_stock;
                        SoStock::where('so_id', $order['user_id'])->where('product_id', $product['product_id'])
                            ->update(array('qty' => $total_stock));
                    }
                }
                if($OrderProduct == '') {
                    $order_products = new OrderProduct();
                    $order_products->order_id = $orderObj->id;
                    $order_products->product_id = $product['product_id'];
                    $order_products->quantity = $product['quantity'];
                    $order_products->product_actual_price = $product['product_actual_price'];
                    $order_products->product_discounted_price = $product['product_discounted_price'];
                    if($orderObj->country_id == 2){
                        $order_products->scheme_free_quantity = 0;
                    }else{
                        $order_products->scheme_free_quantity = $product['scheme_free_quantity'];
                    }
                    $order_products->save();
                    foreach ($product['product_category'] as $product_category) {
                        $order_product_category = new OrderProductCategory();
                        $order_product_category->order_product_id = $order_products->id;
                        $order_product_category->product_category_id = $product_category['product_category_id'];
                        $order_product_category->quantity = $product_category['quantity'];
                        $order_product_category->save();

                    }
                }
                if($OrderProduct != ''){
                    $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                        ->where('product_id',$product['product_id'])
                        ->first();
                    if($orderObj->country_id == 2){
                        $product['scheme_free_quantity'] = 0;
                    }else{
                    }

                    $orderProduct = OrderProduct::where('order_id', $orderObj->id)
                        ->where('product_id',$product['product_id'])
                        ->update(array('quantity' => $product['quantity'],'product_actual_price'=> $product['product_actual_price'],
                        'product_discounted_price'=>$product['product_discounted_price'],
                            'scheme_free_quantity'=>$product['scheme_free_quantity']));
                    $OrderProductCategory2 = OrderProductCategory::where('order_product_id',$OrderProduct->id)
                        ->get();
                    foreach ($OrderProductCategory2 as $data){
                        $Order_Products_category[]= $data['product_category_id'];
                    }
                    foreach ($product['product_category'] as $product_category) {

                        $check_values_category[] = $product_category['product_category_id'];
                        $OrderProductCategory = OrderProductCategory::where('order_product_id',$OrderProduct->id)
                            ->where('product_category_id',$product_category['product_category_id'])
                            ->first();

                        if($OrderProductCategory == '') {
                            $order_product_category = new OrderProductCategory();
                            $order_product_category->order_product_id = $OrderProduct->id;
                            $order_product_category->product_category_id = $product_category['product_category_id'];
                            $order_product_category->quantity = $product_category['quantity'];
                            $order_product_category->save();
                        }
                        if($OrderProductCategory != '')
                        {
                            $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                                ->where('product_id',$product['product_id'])
                                ->first();
                            OrderProductCategory::where('order_product_id', $OrderProduct->id)
                                ->where('product_category_id',$product_category['product_category_id'])
                                ->update(array('quantity' => $product_category['quantity']));
                        }

                    }

                }
//               // points on piece/dozen sales
                $point_based=Product::select('points_based','points_based_value')->where('id',$product['product_id'])->first();
                if(!empty($point_based)){
                    $product_id=$product['product_id'];
                    $orders=$this->where('booked_by',$order['user_id'])
                        ->Where('created_at', 'like', '%' . date("Y-m-d") . '%')
                        ->with(['order_products'=>function($query)use ($product_id){return $query->where('product_id',$product_id)->with(['order_sales_points'=>function($query)use ($product_id){return $query->where('product_id',$product_id)->where('date',date("Y-m-d"));}]);}])
                        ->get();
                    $d_quantity=0;
                    foreach ($orders as $point){
                        if(!empty($point['order_products'][0]['quantity']) && count($point['order_products'][0]['order_sales_points'])<1){
                            $d_quantity=$d_quantity+$point['order_products'][0]['quantity'];
                        }
                    }
//                    echo $d_quantity;
//                    echo 's';
//                    echo $point_based['points_based_value'];
//                    echo 's';
                    $chck=[];
                    if($point_based['points_based']=='Dozen'){
                        if(!empty($point_based['points_based_value']) && $d_quantity>=$point_based['points_based_value']*12){
                            $repeat=intval(($d_quantity)/($point_based['points_based_value']*12));
                            if($repeat!=0){
                                for($a=$repeat;$a>0;$a--){
                                    $chck[]=$a;
                                    $userPointsModel->insertdozensalesPoints($order['user_id'], 9,$orderObj->id,$product['product_id']);
                                }
                            }
                        }

                    }elseif($point_based['points_based']=='Per Piece'){
                        if(!empty($point_based['points_based_value']) && $d_quantity>=$point_based['points_based_value']){
                            $repeat=intval(($d_quantity)/$point_based['points_based_value']);
                            if($repeat!=0){
                                for($a=$repeat;$a>0;$a--){
                                    $userPointsModel->insertdozensalesPoints($order['user_id'], 9,$orderObj->id,$product['product_id']);
                                }
                            }
                        }
                    }
                }
            }
                $new_array = array_diff($Order_Products, $check_values);
//                return $new_array;

                OrderProduct::where('order_id', $orderObj->id)->wherein('product_id', $new_array)->delete();
            if($OrderProduct != '') {
                $new_array2 = array_diff($Order_Products_category, $check_values_category);

                OrderProductCategory::where('order_product_id', $OrderProduct->id)->wherein('product_category_id', $new_array2)->delete();
            }
//            return $new_array;

//          $orderObj->products()->sync($products);
        }
        return $this->getOrderDetails($orderObj->id,$order['job_id']);
    }
    /***
     * Add or Update Expense
     *
     * @param $expense
     * @param int $expenseId
     *
     * @return mixed
     */
     public function addOrUpdateExpense($expense,$expenseId = 0)
    {
        $giveUserPoints = true;
        $userPointsModel = new UserPoint();
        $expenseObj = new UserExpense();
        if (!empty($expense['expense_id'])&& $expense['expense_id']!=0) {
            $expenseObj = UserExpense::where('id', $expense['expense_id'])
                ->first();
            // for credit note only  amount update
            if((!empty($expense['id']))&&(($expense['id']==71) || ($expense['id']==82))){
                $expenseObj->amount = $expense['amount'];
                $expenseObj->status = 'Pending';
            }
            
            $giveUserPoints = false;
            //Expense not found
            if (is_null($expenseObj)) {
                return $expenseObj;
            }
        }

        if ( $expense['expense_id']==0) {
               
            if($expense['amount'] != 0) {
                $expenseObj->uuid = $expense['uuid'];
                $expenseObj->user_id = $expense['user_id'];
                $expenseObj->expense_id = $expense['id'];
                if(!empty($expense['account_from_id'])){
                    $expenseObj->account_from_id = $expense['account_from_id'];
                }
                if(!empty($expense['account_to_id'])){
                    $expenseObj->account_to_id = $expense['account_to_id'];
                }

                $expenseObj->amount = $expense['amount'];
                $expenseObj->date =  date('Y-m-d');
                $expenseObj->saller_name = $expense['saller_name'];
                $expenseObj->saller_phone_no = $expense['saller_phone_no'];
                $expenseObj->invoice_no = $expense['invoice_no'];
                $expenseObj->party_id = $expense['party_id'];
                $expenseObj->remarks = $expense['remarks'];
                $expenseObj->type = $expense['type'];


                $expenseObj->save();
                if(!empty($expenseObj->party_id) && $expenseObj->party_id!=0 ){
                    if($expenseObj->expense_id==71){
                    $expenseObj->status = 'Approved';
                    $pname=PartyName::where('id',$expense['party_id'])->first();
                    if(!empty($pname)){
                      //  $pname->balance=($pname->balance)-($expense['amount']);
                      //  $pname->save();
                        $e= new UserExpenseDetail();
                        $e->user_id=$expense['user_id'];
                        $e->detail_id=$expenseObj->id;
                        $e->party_id=$expense['party_id'];
                        $e->amount=$expense['amount'];
                        $e->balance=$pname->balance;
                        $e->type='Credit Note';
                        $e->date= date('Y-m-d');
                        $e->save();
                        if(!empty($pname->id)){
                            LedgerAdjust($pname->id);
                        }
                    }
                    }else{
                        $expenseObj->status = 'Pending';
                    }
                    $expenseObj->save();
                }else{
                    $userExpenseObj = new UserExpenseDetail();
                    $userExpenseObj->detail_id=$expenseObj->id;
                    if (array_key_exists('type', $expense)) {
                        $userExpenseObj->type = $expense['type'];
                    } else {
                        $userExpenseObj->type = 'Expense';
                    }
                    $user = User::where('id', $expense['user_id'])->first();
                    $old_balance = $user['balance'];
                    $new_balance = $expense['amount'];

                    if(!empty($expense['type']) && $expense['type']=='Recovery'){
                        $total_balance = $old_balance + $new_balance;
                        User::where('id', $expense['user_id'])->update(array('balance' => $total_balance));
                    }else{
                        $total_balance = $old_balance - $new_balance;
                        User::where('id', $expense['user_id'])->update(array('balance' => $total_balance));
                    }
                    $userExpenseObj->user_id = $expense['user_id'];
                    $userExpenseObj->balance = $total_balance;
                    $userExpenseObj->amount = $expense['amount'];
                    $userExpenseObj->date =  date('Y-m-d');
                    $userExpenseObj->save();
                }
                //    $total_balance = $old_balance - $new_balance;
                //    User::where('id', $expense['user_id'])->update(array('balance' => $total_balance));




            }
        }else{
            $expenseObj->invoice_no = $expense['invoice_no'];
            $expenseObj->remarks = $expense['remarks'];
            $expenseObj->save();
        }
//    Image upload
        if(!empty($expense['uploadimages'])) {
            $images = [];
            foreach($expense['uploadimages'] as $image) {
                $image = upload_base64_image($image, 'uploads/userExpense/',
                    'image-');
                $customerImage = new ExpenseImage();
                $customerImage->image =  $image;
                $images[] = $customerImage;
            }
            if(count($images) > 0) {
                $expenseObj->images()->saveMany($images);
            }
        }
         if(!empty($expense['expense_tags'])) {
            $tags = [];
            foreach($expense['expense_tags'] as $tag) {
                $customerImage = new ExpenseTag();
                $customerImage->expense_id=$expenseObj->id;
                $customerImage->tag_id=$tag;
                $tags[] = $customerImage;
            }
            if(count($tags) > 0) {
               $expenseObj->Tags()->saveMany($tags);
            }
        }
        return 'success';
    }
    /***
     * Add Order Invoice
     *
     * @param int $orderId
     *
     * @return mixed
     */
    public function addOrderInvoice($orderId = 0)
    {
        $order = $this->where('id', $orderId)->first();
        if ($order != ''){
            if ($order->invoice_no == '' && $order->booked_by==13 ) {
                $max_num = $this->max('invoice_no');
                $invoice_num = $max_num + 1;
                $order = Order::where('id', $orderId)->update(['invoice_no' => $invoice_num]);
                return $invoice_num;
            }
            if ($order->invoice_no == 0 && $order->booked_by==13) {
                $max_num = $this->max('invoice_no');
                $invoice_num = $max_num + 1;
                $order = Order::where('id', $orderId)->update(['invoice_no' => $invoice_num]);
                return $invoice_num;
            } else {
                return $order->invoice_no;
            }
    }else{
        return 'No Order';
        }
    }
    /***
     * Add or Update Order
     *
     * @param $order
     * @param int $orderId
     *
     * @return mixed
     */
    public function addOrUpdateReturnOrder($order, $orderId = 0)
    {
        $orderObj = new $this();
        if ($orderId > 0) {
            $orderObj = $this->with('products')
                ->where('id', $orderId)
                ->first();
            //Order not found
            if (is_null($orderObj)) {
                return $orderObj;
            }
            $orderObj->updated_by = $order['user_id'];
        } else {
            $user_country_id = User::select('country_id')->where('id',$order['user_id'])->first();
            $orderObj->country_id = $user_country_id->country_id;
            $orderObj->booked_by = $order['user_id'];
            $orderObj->created_by = $order['user_id'];
            $orderObj->agency_id = $order['agency_id'];
        }
        //Contact Person ID

        if (array_key_exists('shop_id', $order)) {
            if($order['shop_id'] != 0){
                $orderObj->shop_id = $order['shop_id'];
            }else{
                $customerModel = new Customer();
                $customerId = $customerModel->getCustomerId($order['customer']);
                $orderObj->shop_id = $customerId;
            }
        } elseif (array_key_exists('customer', $order)) {

            $customerModel = new Customer();
            $customerId = $customerModel->getCustomerId($order['customer']);
            $orderObj->shop_id = $customerId;
        }
        $orderObj->status = 'Returned';
        $orderObj->date_to_deliver = date('Y-m-d');
        $payment = round($order['payment'],2);
        $orderObj->price = $payment*-1;
        $orderObj->uuid = $order['uuid'];
        $orderObj->is_discounted = 0;

        if (array_key_exists('remarks', $order)) {
            $orderObj->remarks = $order['remarks'];
        }

        if (array_key_exists('payment', $order)) {
            $orderObj->payment = $payment*-1;
        }

        if (array_key_exists('delivery_time', $order)) {
            $orderObj->delivery_time = $order['delivery_time'];
        }

        if (array_key_exists('remarks', $order)) {
            $orderObj->remarks = $order['remarks'];
        }

        if (array_key_exists('latitude', $order)) {
            $orderObj->latitude = $order['latitude'];
        }

        if (array_key_exists('longitude', $order)) {
            $orderObj->longitude = $order['longitude'];
        }

        if (array_key_exists('discount', $order)) {
            $orderObj->discount = $order['discount'];
        }
        $orderObj->type = 'Query';

        //Update Ledger
        $customer = Customer::where('id', $order['shop_id'])->first();
        $old_balance = $customer['balance'];
        $new_balance = $order['payment'];
        $total_balance = $old_balance + $new_balance;
        Customer::where('id', $order['shop_id'])->update(array('balance' => $total_balance));
//        Order::where('id', $orderId)->update(array('balance' => $total_balance));
        $orderObj->balance = $total_balance;
        $orderObj->save();
        //Remove Image
        if(array_key_exists('remove_images', $order)) {
            $orderObj->images()->whereIn('image', $order['remove_images'])
                ->delete();
            foreach($order['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/order/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }

        if(array_key_exists('products', $order)) {
            $products = [];
            $Order_Products=[];
            $check_values=[];
            $Order_Products_category=[];
            $check_values_category=[];
            $OrderProduct2 = OrderProduct::where('order_id',$orderObj->id)
                ->get();
            foreach ($OrderProduct2 as $value){
                $Order_Products[]= $value['product_id'];
            }
            foreach($order['products'] as $key =>$product) {
                $check_values[] = $product['product_id'];
                $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                    ->where('product_id',$product['product_id'])
                    ->first();
                    $so_stock = SoStock::where('so_id',$order['user_id'])->where('product_id',$product['product_id'])->first();
                    if($so_stock != '') {
                        $old_stock = $so_stock['qty'];
                        $new_stock = $product['quantity'];
                        $total_stock = $old_stock + $new_stock;
                        SoStock::where('so_id', $order['user_id'])->where('product_id', $product['product_id'])
                            ->update(array('qty' => $total_stock));
                    }
                if($OrderProduct == '') {
                    $order_products = new OrderProduct();
                    $order_products->order_id = $orderObj->id;
                    $order_products->product_id = $product['product_id'];
                    $order_products->quantity = $product['quantity'];
                    $order_products->product_actual_price = $product['product_actual_price'];
                    $order_products->product_discounted_price = $product['product_discounted_price'];
                    if($orderObj->country_id == 2){
                        $order_products->scheme_free_quantity = 0;
                    }else{
                        $order_products->scheme_free_quantity = $product['scheme_free_quantity'];
                    }
                    $order_products->save();
                    foreach ($product['product_category'] as $product_category) {
                        $order_product_category = new OrderProductCategory();
                        $order_product_category->order_product_id = $order_products->id;
                        $order_product_category->product_category_id = $product_category['product_category_id'];
                        $order_product_category->quantity = $product_category['quantity'];
                        $order_product_category->save();

                    }
                }
                if($OrderProduct != ''){
                    $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                        ->where('product_id',$product['product_id'])
                        ->first();
                    if($orderObj->country_id == 2){
                        $product['scheme_free_quantity'] = 0;
                    }else{
                    }
                    $orderProduct = OrderProduct::where('order_id', $orderObj->id)
                        ->where('product_id',$product['product_id'])
                        ->update(array('quantity' => $product['quantity'],'product_actual_price'=> $product['product_actual_price'],
                        'product_discounted_price'=>$product['product_discounted_price'],
                            'scheme_free_quantity'=>$product['scheme_free_quantity']));
                    $OrderProductCategory2 = OrderProductCategory::where('order_product_id',$OrderProduct->id)
                        ->get();
                    foreach ($OrderProductCategory2 as $data){
                        $Order_Products_category[]= $data['product_category_id'];
                    }
                    foreach ($product['product_category'] as $product_category) {

                        $check_values_category[] = $product_category['product_category_id'];
                        $OrderProductCategory = OrderProductCategory::where('order_product_id',$OrderProduct->id)
                            ->where('product_category_id',$product_category['product_category_id'])
                            ->first();

                        if($OrderProductCategory == '') {
                            $order_product_category = new OrderProductCategory();
                            $order_product_category->order_product_id = $OrderProduct->id;
                            $order_product_category->product_category_id = $product_category['product_category_id'];
                            $order_product_category->quantity = $product_category['quantity'];
                            $order_product_category->save();
                        }
                        if($OrderProductCategory != '')
                        {
                            $OrderProduct = OrderProduct::where('order_id',$orderObj->id)
                                ->where('product_id',$product['product_id'])
                                ->first();
                            OrderProductCategory::where('order_product_id', $OrderProduct->id)
                                ->where('product_category_id',$product_category['product_category_id'])
                                ->update(array('quantity' => $product_category['quantity']));
                        }

                    }

                }
            }

                $new_array = array_diff($Order_Products, $check_values);
                OrderProduct::where('order_id', $orderObj->id)->wherein('product_id', $new_array)->delete();
            if($OrderProduct != '') {
                $new_array2 = array_diff($Order_Products_category, $check_values_category);

                OrderProductCategory::where('order_product_id', $OrderProduct->id)->wherein('product_category_id', $new_array2)->delete();
            }
        }
        return $this->getOrderDetails($orderObj->id,$order['job_id']);
    }
    /***
     *  Update Bilty
     *
     * @param $bilty
     * @param int $biltyId
     *
     * @return mixed
     */
    public function updateBilty($bilty, $biltyId = 0)
    {
        $giveUserPoints = true;
        $userPointsModel = new UserPoint();
        $orderObj = new $this();
        $today = date('Y-m-d');
        $update_bilty = Bilti::where('id', $biltyId)
            ->update(array('bilti_received_date' => $today,'status'=>'Received'));
        foreach ($bilty['products'] as $product){
            $update_bilty_detail = BiltiDetail::where('bilti_id', $biltyId)->where('product_id',$product['id'])
                ->update(array('quantity_received' => $product['quantity']));
            foreach ($product['product_category'] as $product_category){
                $bityDetail = BiltiDetail::where('bilti_id', $biltyId)->where('product_id',$product['id'])->first();
                $update_bilty_detail_type = BiltiDetialType::where('bilti_detial_id', $bityDetail->id)
                    ->where('product_category_id',$product_category['id'])
                    ->update(array('quantity_received' => $product_category['quantity']));
            }
            $so_stock = SoStock::where('so_id',$bilty['user_id'])->where('product_id',$product['id'])->first();
            if($so_stock != '') {
                $old_stock = $so_stock['qty'];
                $new_stock = $product['quantity'];
                $total_stock = $old_stock + $new_stock;
                SoStock::where('so_id', $bilty['user_id'])->where('product_id', $product['id'])
                    ->update(array('qty' => $total_stock));
            }else{
               $stock = new SoStock();
                $stock->so_id = $bilty['user_id'];
                $stock->product_id = $product['id'];
                $stock->qty = $product['quantity'];
                $stock->save();
            }
        }

        return $update_bilty_detail;
    }
    /***
     * Add or Update Order Image
     *
     * @param $order
     * @param int $orderId
     *
     * @return mixed
     */
    public function addOrUpdateOrderImage($order, $orderId = 0)
    {

        $orderObj = new $this();
        if ($orderId > 0) {
            $orderObj = $this->where('id', $orderId)
                ->first();

            //Order not found
            if (is_null($orderObj)) {
                return $orderObj;
            }
        }
        //Image upload
        if (array_key_exists('images', $order)) {
            $images = [];
            foreach ($order['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/order/',
                    'image-');
                $orderImage = new OrderImage();
                $orderImage->image = 'http://' . $image;
                $images[] = $orderImage;
            }
            if (count($images) > 0) {
                $orderObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $order)) {
            $orderObj->images()->whereIn('image', $order['remove_images'])
                ->delete();
            foreach($order['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/order/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return $this->getOrderImage($orderObj->id);
    }
    /***
     * Get Order Images
     *
     * @param $id
     * @return mixed
     */
    public function getOrderImage($id)
    {
        $order = OrderImage::
        where('order_id', $id)
            ->orderBy('id', 'desc')
            ->first();
        return $order;
    }



    /***
     * Get Order Details
     *
     * @param $id
     * @return \Illuminate\Database\Query\Builder
     */
    public function getOrderDetails($id,$job_id)
    {
         $order = $this->with(['products' => function($query) {
                                    $query->with(['images' => function ($query) {
                                        $query->get();}]);
                        }, 'images','customer' => function($query){
                                    $query->with('images', 'contactPerson', 'proprietor','product_discount');
        }])->where('id', $id)
           ->first();
        $obj2=null;
            foreach ($order->products as $item){
                $order_product_category = '';
                if (isset($item->pivot->id))
                    $order_product_category = OrderProductCategory::where('order_product_id', $item->pivot->id)
                        ->with('product_category_info')->get();
                if(count($order_product_category)>0){
                    foreach ($order_product_category as $info){
                        foreach ($info->product_category_info as $data)
                            $info->name =  $data->name;
                        unset($info->product_category_info);
                    }
                }
                $item->product_category = $order_product_category;
                $obj2[] = $item;
            }
            $obj1 = $order;
            $obj1->products = $obj2;
            $value = $obj1;
//         $order = $this->with(['products' => function($query) {
//                                    $query->with('images');
//                        }])->where('id', $id)
//           ->first();
        $order['job_id'] = $job_id;
        return $order;
    }
    public function getAreaOrders($area_id,$so_id)
    {
         $order = $this->area_orders()->where('sale_officer_id',$so_id)->get();
        return $order;
    }
    public function getOrderDetail($uuId)
    {
         $order = $this->where('uuid', $uuId)
           ->first();
//         $order = $this->with(['products' => function($query) {
//                                    $query->with('images');
//                        }])->where('id', $id)
//           ->first();
        if(is_null($order)){
            return 'Order not found';
        }
        return $order;
    }
    /***
     * Get SO Orders
     *
     * @param $id
     * @return \Illuminate\Database\Query\Builder
     */
    public function getSoOrders($id)
    {
                 $orders = $this
             ->where('booked_by', $id)
             ->where('status','!=', 'Cancelled')
             ->where('status','!=', 'Delivered')
             ->where('status','!=', 'Rejected')
           ->get();
        $main_data = array();
        foreach ($orders as $value){
            if(isset($value['shop_id']) && $value['shop_id']!=''){
                $customer_name=Customer::select('name')->where('id',$value['shop_id'])->first();
                $customer_image=CustomerImage::select('image_url')->where('shop_id',$value['shop_id'])->first();
                $value['customer_name']=$customer_name['name'];
                $value['customer_image']=$customer_image['image_url'];
            }
            $main_data[]=$value;
        }

        return $main_data;
    }
    /***
     * Get Countries
     *
     * @return \Illuminate\Database\Query\Builder
     */
    public function getCountries()
    {
                 $countries = Country::with('currencies')->get();
        return $countries;
        $main_data = array();
        foreach ($orders as $value){
            if(isset($value['shop_id']) && $value['shop_id']!=''){
                $customer_name=Customer::select('name')->findorfail($value['shop_id'])->toarray();
                $customer_image=CustomerImage::select('image_url')->where('shop_id',$value['shop_id'])->first();
                $value['customer_name']=$customer_name['name'];
                $value['customer_image']=$customer_image['image_url'];
            }
            $main_data[]=$value;
        }

        return $main_data;
    }

    /***
     * Order list
     * @param $userId
     * @param $avoidPagination
     * @param int $page
     * @return \Illuminate\Database\Eloquent\Collection|static[]
     */
    public function getOrders($userId, $avoidPagination, $page = 1)
    {

        $query = $this->with(['images','products' => function($query) {
                            $query->with('images');
        },      'customer' => function($query){
                        $query->with('images', 'contactPerson', 'proprietor');
        }]);
        if($userId > 0) {
            $query->where('booked_by', $userId);
            $user = User::where('id',$userId)->first();
            $query->where('agency_id', $user->agency_id);

        }
        if($page > 0 && !$avoidPagination) {
            $offset = calculate_offset($page);
            $query->skip($offset)
                  ->take(10);
        }
        return $query->get();
    }
    /***
     * Get Order ID using uuid
     *
     * @param $uuid
     * @return int
     */
    public function getOrderId($uuid)
    {
        $orderRecord =  $this->where('uuid', $uuid)
            ->first();
        return is_null($orderRecord) ? 0 : $orderRecord->id;
    }

    /***
     * Get total orders amount
     *
     * @param $customerId
     * @return mixed
     */
    public function getTotalOrdersAmount($customerId)
    {
        return $this->where('customer_id', $customerId)
                    ->sum('price');
    }

    /***
     * Update order status
     *
     * @param $orderId
     * @param $userId
     * @param $status
     * @param $remarks
     *
     * @return object
     *
     */
    public function updateOrderStatus($orderId, $userId, $status, $remarks = null)
    {
        $updateData = [
                        'updated_by' => $userId,
                        'status'     => $status
                      ];
        if(!is_null($remarks)) {
            $updateData['remarks'] = $remarks;
        }

        $this->where('id', $orderId)
             ->update($updateData);

        return $this->getOrderDetails($orderId,0);
    }
    public function statusOrder($orderId,$status,$invoice_no,$user_id)
    {
        if($status == 'Delivered'){
            $updateData = [
                'status' => $status,
//                'invoice_no' => $invoice_no,
                'date_to_deliver' => date('y-m-d')
            ];
        }else{
            $updateData = [
                'status' => $status,
//                'invoice_no' => $invoice_no,
            ];
        }


        $this->where('id', $orderId)
            ->update($updateData);

        if ($status == 'Delivered') {
        $order = Order::where('id', $orderId)->first();

        $customer = Customer::where('id', $order['shop_id'])->first();
        $old_balance = $customer['balance'];
        $new_balance = $order['payment'];
        $total_balance = $old_balance - $new_balance;
        Customer::where('id', $order['shop_id'])->update(array('balance' => $total_balance));
            Order::where('id', $orderId)->update(array('balance' => $total_balance));

            $OrderProduct = OrderProduct::where('order_id',$orderId)
                ->get();
            foreach($OrderProduct as $key =>$product) {
                $so_stock = SoStock::where('so_id',$user_id)->where('product_id',$product['product_id'])->first();
                if($so_stock != '') {
                    $old_stock = $so_stock['qty'];
                    $new_stock = $product['quantity'];
                    $total_stock = $old_stock - $new_stock;
                    SoStock::where('so_id', $user_id)->where('product_id', $product['product_id'])
                        ->update(array('qty' => $total_stock));
                }
            }
            $order = Order::where('id', $orderId)->with(['products'])->first();
            $order_invoice= $order->invoice_no;
            $order_products = $order->products;
                if (empty($order_invoice) && $user_id==13 ) {
//                    $max_num = $order->max('invoice_no');
//                    $max_num = $order->max('invoice_no');
                    $max_num =  DB::table('orders')->max('invoice_no');
//                    dd($max_num+1) ;
//                    $max_num = Order::find(DB::table('orders')->max('invoice_no'));
                    $invoice_num = $max_num + 1;
                    $order = Order::where('id', $orderId)->update(['invoice_no' => $invoice_num]);
                }else if ( $order_invoice == 0 && $user_id==13) {
//                    $max_num = $order->max('invoice_no');
//                    $max_num = Order::find(DB::table('orders')->max('invoice_no'));
                    $max_num =  DB::table('orders')->max('invoice_no');
                    $invoice_num = $max_num + 1;
                    $order = Order::where('id', $orderId)->update(['invoice_no' => $invoice_num]);
                }

            $old_balance = $old_balance*-1;
            $total_balance = $total_balance*-1;
            $customer = Customer::where('id', $order['shop_id'])->first();
            $customer_mail = $customer['email'];
            $customer_name = $customer['name'];
            $date = date('Y-m-d');
//            if(empty($customer_mail) || url('')=='http://demo.biocospk.com') {
//                Mail::send('email.deliver_order', compact('old_balance', 'new_balance','total_balance','date','customer_name','order_products'), function($message) use ($customer_mail){
//                    $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
//                    $message->to('ntariq@xynoapps.com')->subject('Order Delivered!');
//                });
//            }else{
//                Mail::send('email.deliver_order', compact('old_balance', 'new_balance','total_balance','date','customer_name','order_products'), function($message) use ($customer_mail){
//                    $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
//                    $message->to($customer_mail)->subject('Order Delivered!');
//                });
//            }
        }

        return $this->getOrderDetails($orderId,0);
    }
    public function deliverOrder($uuId,$status)
    {
        if($status == 'Delivered'){
            $updateData = [
                'status' => $status,
                'date_to_deliver' => date('y-m-d')
            ];
        }else{
            $updateData = [
                'status' => $status,
            ];
        }

        $this->where('uuid', $uuId)
            ->update($updateData);

        if ($status == 'Delivered') {
        $order = Order::where('uuid', $uuId)->first();

        $customer = Customer::where('id', $order['shop_id'])->first();
        $old_balance = $customer['balance'];
        $new_balance = $order['payment'];
        $total_balance = $old_balance - $new_balance;
        Customer::where('id', $order['shop_id'])->update(array('balance' => $total_balance));
            $order->update(array('balance' => $total_balance));

        }

        return $this->getOrderDetail($uuId);
    }

    /***
     * Get orders for customer
     *
     * @param $customerId
     * @return mixed
     */
    public function getOrdersForCustomer($customerId)
    {
        return $this->with(['payments'])
//        return $this->with(['payments' => function($query){
//                            $query->where('is_success', 1);
//                    }])
//                    ->where('customer_id', $customerId)
                    ->where('shop_id', $customerId)
                    ->whereNotIn('status', ['Rejected','Cancelled'])
                    ->get();
    }
}