<?php

namespace App\Models;


use Illuminate\Database\Eloquent\Model;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Mail;

class PaymentReceived extends Model
{
    /**
     * The table name
     *
     * @var string
     */
    protected $table = 'payments';
    /***
     * The attributes that are mass assignable
     *
     * @var array
     */
    protected $guarded = ['id'];
    /***
     * Timestamp false
     *
     * @var bool
     */
    public $timestamps = false;

    /***
     * Validation Rules
     *
     * @param null $attributes
     * @return array
     */
    public function validationRules($attributes = null)
    {
        $rules = [
            'uuid'            => 'required',
            'customer_id'     => 'required',
            'payment_type'    => ['required', Rule::in(['Promise','Cheque','Cash']) ],
            'amount'          => 'required',
            'cheque_type'     => [Rule::in(['Bearer Cheque','Order Cheque','Crossed Cheque',
                                            'Account Payee Cheque','Company Crossed Cheque',
                                            'Stale Cheque','Post Dated Cheque','Anti Dated Cheque' ])]
        ];

        // no list is provided
        if(!$attributes)
            return $rules;

        // a single attribute is provided
        if(!is_array($attributes))
            return [ $attributes => $rules[$attributes] ];

        // a list of attributes is provided
        $newRules = [];
        foreach ( $attributes as $attr )
            $newRules[$attr] = $rules[$attr];
        return $newRules;
    }

    /***
     * Add or Update Payment Received
     *
     * @param $paymentReceived
     * @return Model|null|static
     */
    public function addOrUpdatePaymentReceived($paymentReceived)
    {
        $giveUserPoints = true;
        $paymentReceivedObj = new $this();
        if(($paymentReceived['uuid']) != '') {
            $paymentReceivedObj = $this->where('uuid', $paymentReceived['uuid'])
                                       ->first();
            $giveUserPoints = false;
            //Payment Received not found
            if(is_null($paymentReceivedObj)) {
                $paymentReceivedObj = new   $this();
                $paymentReceivedObj->user_id     = $paymentReceived['user_id'];
            }
        } else {
            $paymentReceivedObj->user_id     = $paymentReceived['user_id'];
        }
        $paymentReceivedObj->uuid           = $paymentReceived['uuid'];
        $paymentReceivedObj->shop_id    = $paymentReceived['shop_id'];
        $paymentReceivedObj->agency_id    = $paymentReceived['agency_id'];
        $paymentReceivedObj->payment_type   = $paymentReceived['payment_type'];
        $paymentReceivedObj->amount         = $paymentReceived['amount'];
        $paymentReceivedObj->created_at         =  date('Y-m-d');
//        $paymentReceivedObj->bank_id         = $paymentReceived['bank_id'];
        if($paymentReceivedObj->order_id <= 0) {
            $paymentReceivedObj->order_id = $this->getOrderIdForCustomer($paymentReceived['shop_id'],
                                                    $paymentReceived['amount']);
        }
        //Remarks on payment received
        if(array_key_exists('remarks', $paymentReceived)) {
            $paymentReceivedObj->remarks = $paymentReceived['remarks'];
        }
        // Promise Cheque Date
        if(array_key_exists('promise_cheque_date', $paymentReceived)) {
            $paymentReceivedObj->promise_cheque_date = $paymentReceived['promise_cheque_date'];
        }else{
            $paymentReceivedObj->promise_cheque_date = $paymentReceived['visit_date'];

        }
        // Cheque Type
        if(array_key_exists('cheque_type', $paymentReceived)) {
            $paymentReceivedObj->cheque_type = $paymentReceived['cheque_type'];
        }
        // Cheque Number
        if(array_key_exists('cheque_no', $paymentReceived)) {
            $paymentReceivedObj->cheque_no = $paymentReceived['cheque_no'];
        }

        // IS Success
//        if(array_key_exists('is_success', $paymentReceived)) {
//            $paymentReceivedObj->is_success = $paymentReceived['is_success'];
//        }
        $paymentReceivedObj->save();

        // Delete Promise
        if(array_key_exists('promise_id', $paymentReceived)) {
            $this->where('id', $paymentReceived['promise_id'])
                ->delete();
        }
        //update customer balance detail
        if($paymentReceived['payment_type'] != 'Promise' && $paymentReceived['payment_type'] != 'Cheque' ) {

            $Role = User::find( $paymentReceived['user_id'] )->roles->first()->name ;
            if($Role=='so' ||$Role=='su'){
                $target=null;
                $cus_so=Customer::where('id',$paymentReceived['shop_id'])->first();
                if(!empty($cus_so) && !empty($cus_so['sale_officer_id']))
                $target=TargetDetail::where('date', 'like', '%' . date("Y-m") . '%')->where('user_id',$cus_so['sale_officer_id'])->first();
                if(!empty($target)){
                    $recovery=$this->where('user_id', $paymentReceived['user_id'])
                        ->where('promise_cheque_date', 'like', '%' . date("Y-m") . '%')
                        ->sum('amount');
                    if($recovery>=$target['recovery_target']){
                        $userPointsModel = new UserPoint();;
                        $userPointsModel->insertUserPoints($cus_so['sale_officer_id'], 4,$paymentReceivedObj->id);
                    }
                }
            }


            $customer = Customer::where('id', $paymentReceived['shop_id'])->first();
            $old_balance = $customer['balance'];
            $new_balance = $paymentReceived['amount'];
            $total_balance = $old_balance + $new_balance;
            Customer::where('id', $paymentReceived['shop_id'])->update(array('balance' => $total_balance));
            $paymentReceivedObj->update(array('balance' => $total_balance));

            //update user balance detail

            $user = User::where('id', $paymentReceived['user_id'])->first();
            $old_balance_user = $user['balance'];
            $new_balance = $paymentReceived['amount'];
            $total_balance_user = $old_balance_user + $new_balance;
            User::where('id', $paymentReceived['user_id'])->update(array('balance' => $total_balance_user));

            $userExpenseObj = new UserExpenseDetail();
            $userExpenseObj->type = 'Recovery';
            $userExpenseObj->user_id = $paymentReceived['user_id'];
            $userExpenseObj->detail_id = $paymentReceivedObj->id;
            $userExpenseObj->balance = $total_balance_user;
            $userExpenseObj->amount = $paymentReceived['amount'];
            if(array_key_exists('promise_cheque_date', $paymentReceived)) {
                $userExpenseObj->date = $paymentReceived['promise_cheque_date'];
            }else{
                $userExpenseObj->date = $paymentReceived['visit_date'];

            }
//            $userExpenseObj->date = $expense['date'];
            $userExpenseObj->save();




            $old_balance = $old_balance*-1;
            $total_balance = $total_balance*-1;
            $customer = Customer::where('id', $paymentReceived['shop_id'])->first();
            $customer_mail = $customer['email'];
            $customer_name = $customer['name'];
            $date = date('Y-m-d');

//            if(empty($customer_mail) || url('')=='http://demo.biocospk.com') {
//                Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
//                    $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
//                    $message->to('ntariq@xynoapps.com')->subject('Payment Received!');
//                });
//            }else{
//                Mail::send('email.payment_received', compact('old_balance', 'new_balance','total_balance','date','customer_name'), function($message) use ($customer_mail){
//                    $message->from('admin@biocospk.com', 'Biocos International Pvt Ltd');
//                    $message->to($customer_mail)->subject('Payment Received!');
//                });
//            }
        }
        //add job detail
        $job_detail = JobDetail::where('job_id', $paymentReceived['job_id'])
            ->where('shop_id', $paymentReceived['shop_id'])
            ->first();
        if ($job_detail == '') {
            $job_detail = new JobDetail();
            $job_detail->uuid = $paymentReceived['uuid'];
            $job_detail->job_id = $paymentReceived['job_id'];
            $job_detail->shop_id = $paymentReceived['shop_id'];
            $job_detail->visit_date = $paymentReceived['visit_date'];
            $job_detail->job_type = 'payment';
            $job_detail->job_type_id = $paymentReceivedObj->id;
            $job_detail->comment = $paymentReceived['remarks'];
            $job_detail->reason = '';
            $job_detail->latitude = $paymentReceived['latitude'];
            $job_detail->longitude = $paymentReceived['longitude'];
            $job_detail->save();

            //Image upload
            if (array_key_exists('images', $paymentReceived)) {
                $images = [];
                foreach ($paymentReceived['images'] as $image) {
                    $image = upload_base64_image($image, 'uploads/jobs/',
                        'image-');
                    $JobDetailImage = new JobDetailImage();
                    $JobDetailImage->image_url = 'http://' . $image;
                    $images[] = $JobDetailImage;
                }
                if (count($images) > 0) {
                    $job_detail->images()->saveMany($images);
                }
            }
        }else{

            if ($job_detail->job_type == 'no-order' || $job_detail->job_type == 'order') {

                $job_detail = new JobDetail();
                $job_detail->uuid = $paymentReceived['uuid'];
                $job_detail->job_id = $paymentReceived['job_id'];
                $job_detail->shop_id = $paymentReceived['shop_id'];
                $job_detail->visit_date = $paymentReceived['visit_date'];
                $job_detail->job_type = 'payment';
                $job_detail->job_type_id = $paymentReceivedObj->id;
                $job_detail->comment = $paymentReceived['remarks'];
                $job_detail->reason = '';
                $job_detail->latitude = $paymentReceived['latitude'];
                $job_detail->longitude = $paymentReceived['longitude'];
                $job_detail->save();

                //Image upload
                if (array_key_exists('images', $paymentReceived)) {
                    $images = [];
                    foreach ($paymentReceived['images'] as $image) {
                        $image = upload_base64_image($image, 'uploads/jobs/',
                            'image-');
                        $JobDetailImage = new JobDetailImage();
                        $JobDetailImage->image_url = 'http://' . $image;
                        $images[] = $JobDetailImage;
                    }
                    if (count($images) > 0) {
                        $job_detail->images()->saveMany($images);
                    }

                }

//                $job = JobDetail::where('job_id', $paymentReceived['job_id'])
//                    ->where('shop_id', $paymentReceived['shop_id'])
//                    ->update(array('job_type' => 'payment'));

            }
            elseif ($job_detail->job_type == 'revisit') {
                JobDetail::where('job_id', $paymentReceived['job_id'])
                    ->where('shop_id', $paymentReceived['shop_id'])
                    ->where('job_type', 'revisit')
                    ->update(array('job_type' => 'payment'));
            }
        }

        //Image upload
        if(array_key_exists('images', $paymentReceived)) {
            $images = [];
            foreach($paymentReceived['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/payment/',
                    'paymentimage-');
                $paymentImage = new PaymentImage();
                $paymentImage->image = 'http://'.$image;
                $images[] = $paymentImage;
            }
            if(count($images) > 0) {
                $paymentReceivedObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $paymentReceived)) {
            $paymentReceivedObj->images()->whereIn('image', $paymentReceived['remove_images'])
                ->delete();
            foreach($paymentReceived['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/payment/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }

        return $this->getPaymentReceivedDetails($paymentReceivedObj->id);
    }
    /**
     * Get the user that owns the payment.
     */
    public function user()
    {
        return $this->belongsTo('App\Models\User','user_id');
    }

    public function customer()
    {
        return $this->belongsTo('App\Models\Customer','shop_id','id');
    }
    public function bank_info() {
        return $this->belongsTo('App\Models\Bank','bank_id');

    }

    /***
     * Get Payment Received Details
     *
     * @param $id
     * @return mixed
     */
    public function getPaymentReceivedDetails($id)
    {
        return $this->with('images')
                    ->where('id', $id)
                    ->first();
    }

    /***
     * Get Total amount paid by customer
     * @param $customerId
     * @return mixed
     */
    public function getTotalPaidAmountByCustomer($customerId)
    {
        return $this->where('customer_id', $customerId)
                    ->where('is_success', true)
                    ->sum('amount');
    }
    /***
     * Images of customer/shop
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function images()
    {
        return $this->hasMany('App\Models\PaymentImage', 'payment_id');
    }

    /***
     * Get payment received records
     *
     * @param $userId
     * @param int $customerId
     * @param bool $avoidPagination
     * @param int $page
     * @return mixed
     */
    public function getPaymentReceived($userId, $customerId = 0, $page = 1, $avoidPagination)
    {
        $model = $this->where('user_id', $userId);
        if($customerId > 0) {
            $model->where('customer_id', $customerId);
        }
        $user = User::where('id',$userId)->first();
        $model->where('agency_id', $user->agency_id);
        if($page > 0 && !$avoidPagination) {
            $offset = calculate_offset($page);
            $model->skip($offset)
                ->take(10);
        }
        return $model->get();
    }

    /***
     * Get Total payment made for order
     *
     * @param $orderId
     * @return mixed
     */
    public function getTotalPaymentAgainstOrder($orderId)
    {
        return $this->where('order_id', $orderId)
                      ->where('is_success', true)
                      ->sum('amount');
    }

    /***
     * Get Order Id for Customer
     *
     * @param $customerId
     * @param $currentPaymentAmount
     *
     * @return integer $orderId
     *
     */
    public function getOrderIdForCustomer($customerId, $currentPaymentAmount)
    {
        $orderModel = new Order();
        $orderId = 0;
        $allCustomerOrders = $orderModel->getOrdersForCustomer($customerId);
        foreach($allCustomerOrders as $customerOrder)
        {
            if(count($customerOrder->payments) > 0) {
                $totalPaymentForOrder = $customerOrder->payments->sum('amount');
                $remainingAmount = $customerOrder->price - $totalPaymentForOrder;
                if ($remainingAmount > 0) {
                    $orderId = $customerOrder->id;
                    break;
                }
            } else {
                $orderId = $customerOrder->id;
            }
        }
        return $orderId;
    }
    /***
     * Get Payment ID using different params
     *
     * @param $paymentArray
     * @return int
     */
    public function getPaymentId($paymentArray)
    {
        $paymentRecord =  $this->where('uuid', $paymentArray['uuid'])
            ->first();
        return is_null($paymentRecord) ? 0 : $paymentRecord->id;
    }
    /***
     * Add or Update Payment Image
     *
     * @param $payment
     * @param int $paymentId
     *
     * @return mixed
     */
    public function addOrUpdatePaymentImage($payment, $paymentId = 0)
    {

        $paymentObj = new $this();
        if ($paymentId > 0) {
            $paymentObj = $this->where('id', $paymentId)
                ->first();

            //Order not found
            if (is_null($paymentObj)) {
                return $paymentObj;
            }
        }
        if(array_key_exists('images', $payment)) {
            $images = [];
            foreach($payment['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/payment/',
                    'paymentimage-');
                $paymentImage = new PaymentImage();
                $paymentImage->image = 'http://'.$image;
                $images[] = $paymentImage;
            }
            if(count($images) > 0) {
                $paymentObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $payment)) {
            $paymentObj->images()->whereIn('image', $payment['remove_images'])
                ->delete();
            foreach($payment['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/payment/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return $this->getPaymentImage($paymentObj->id);
    }
    /***
     * Get Payment Images
     *
     * @param $id
     * @return mixed
     */
    public function getPaymentImage($id)
    {
        $payment = PaymentImage::
        where('payment_id', $id)
            ->orderBy('id', 'desc')
            ->first();
        return $payment;
    }
}