<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */

    protected $table = 'products';

    protected $guarded = ['id'];
    /***
     * Images of Product
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function productBookingDetails()
    {
        return $this->hasMany('App\Models\StockBookingDetail', 'product_id','id');
    }
    public function images()
    {
        return $this->hasMany('App\Models\ProductImage', 'product_id');
    }
    public function productPrice()
    {
        return $this->hasMany('App\Models\ProductPrice', 'product_id');
    }
    public function productCategory()
    {
        return $this->hasMany('App\Models\ProductCategory', 'product_id');
    }

    public function raw_product()
    {
        return $this->hasMany('App\Models\ProductRaw', 'product_id');
    }

    /***
     * Get products based on filters
     *
     * @param $agencyId
     * @param $page
     * @param $avoidPagination
     * @return mixed
     */
    public function getProducts($agencyId, $avoidPagination, $page = 1)
    {
        $query = $this->where('agency_id',$agencyId)->with('images')
                      ->whereDate('started_on', '<=', Carbon::now())
                      ->where(function($query){
                          $query->whereDate('discontinued_on', '>', Carbon::now())
                                ->orWhereNull('discontinued_on');
                      });
//        if($agencyId > 0) {
//            $query->where('agency_id', '=', $agencyId);
//        }

        if($page > 0 && !$avoidPagination) {
            $offset = calculate_offset($page);
            $query->skip($offset)
                ->take(10);
        }

        $shops =  $query->get();
        return $shops;
    }
    public function getProductsPrices($agencyId,$userrole)
    {
        if($userrole == "super_admin" || $userrole == "admin" || $userrole == "Accountant" || $userrole == "Export"){
        $products = $this->where('category_id',1)->where('show_product',0)->where('agency_id',$agencyId)->with('images','productPrice','productCategory')->get();
    }else{
        $products = $this->where('category_id',1)->where('show_product',0)->where('show_rsm',0)->where('agency_id',$agencyId)->with('images','productPrice','productCategory')->get();

    }
      
        $product_discounts = ProductDiscount::where('agency_id',$agencyId)->get();
        $discounted_price = [];
        foreach ($products as $key =>$product){
            if(count($product->productPrice) > 0) {
                foreach ($product->productPrice as $data=>$product_price) {
                    $currency = Currency::where('id',$product_price->currency_id)->first();
                foreach ($product_discounts as $value =>$discount) {

//                    $percent_discount = $discount->value/100;

//                    $percent_price = ($product_price->retail_price + $currency->mpx)+($discount->value)  ;
                    if($currency->name == 'PKR'){
//                        $retail_price = ($product_price->retail_price * 12);
                        $retail_price = ($product_price->retail_price );
//                        $retail_price = round($retail_price, 2);
                        $wholesale_price = ($product_price->wholesale_price);
//                        $wholesale_price = round($wholesale_price, 2);
                            $distributor_price = ($product_price->distributor_price );
//                        $distributor_price = round($distributor_price, 2);

                    }elseif ($currency->name == 'AED'){
                        $retail_price = (($product_price->retail_price * 12)+($discount->value))/12  ;
                        $retail_price = round($retail_price, 2);
                        $wholesale_price = (($product_price->wholesale_price * 12)+($discount->value))/12  ;
                        $wholesale_price = round($wholesale_price, 2);
                        $distributor_price = (($product_price->distributor_price * 12)+($discount->value))/12  ;
                        $distributor_price = round($distributor_price, 2);
                    }

                    if($product_price->retail_price == 0){
                        $discounted_price[$data][$value]['retail_price'] = $product_price->retail_price ;

                    }else{
                        if($discount->grade == 'D'){
                            if ($currency->name == 'AED'){
                                $discounted_price[$data][$value]['retail_price'] = $product_price->retail_price ;
                            }if($currency->name == 'PKR'){
                                $discounted_price[$data][$value]['retail_price'] = $retail_price;
                            }
                        }else{
                            $discounted_price[$data][$value]['retail_price'] = $retail_price;
                        }
                    }
                    if($product_price->wholesale_price == 0){
                        $discounted_price[$data][$value]['wholesale_price'] = $product_price->wholesale_price ;
                    }
                    else{
                        if($discount->grade == 'D'){
                            if ($currency->name == 'AED'){
                                $discounted_price[$data][$value]['wholesale_price'] = $product_price->wholesale_price ;
                            }if($currency->name == 'PKR'){
                                $discounted_price[$data][$value]['wholesale_price'] = $wholesale_price;
                            }
                        }else{
                            $discounted_price[$data][$value]['wholesale_price'] = $wholesale_price;
                        }
                    }
                    if($product_price->distributor_price == 0){
                        $discounted_price[$data][$value]['distributor_price'] = $product_price->distributor_price ;
                    }
                    else{
                        if($discount->grade == 'D'){
                            if ($currency->name == 'AED'){
                                $discounted_price[$data][$value]['distributor_price'] = $product_price->distributor_price ;
                            }if($currency->name == 'PKR'){
                                $discounted_price[$data][$value]['distributor_price'] = $distributor_price;
                            }
                        }else{
                            $discounted_price[$data][$value]['distributor_price'] = $distributor_price;
                        }
                    }

                    $discounted_price[$data][$value]['value'] = $discount->value;

                    $discounted_price[$data][$value]['product_discount_id'] = $discount->id;
                    $discounted_price[$data][$value]['grade'] = $discount->grade;
                    $discounted_price[$data][$value]['currency_name'] = $currency->name;
                    $discounted_price[$data][$value]['mpx'] = $currency->mpx;
//                    $discounted_price[$data][$value]['product_retail_price'] = $product_price->retail_price;
//                    $discounted_price[$data][$value]['product_wholesale_price'] = $product_price->wholesale_price;
                }

                }
            }
            $dis = [];
            foreach ($discounted_price as $dist){
                foreach ($dist as $dist2){
                    $dis[]=$dist2;
                }
            }

            $products[$key]['discounted_price'] =  $dis;
//            $products[$key]['discounted_price'] =  $discounted_price;

        }
        return $products;
    }

    /***
     * Get product detail using id
     *
     * @param $productId
     * @return mixed
     */
    public function getProductDetail($productId)
    {
        $product = $this->with('images')
                        ->where('id', '=', $productId)
                        ->first();
        return $product;

    }
}