<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;

class UserExpense extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the Agency that owns the Area.
     */
    public function agency()
    {
        return $this->belongsTo('App\Agency');
    }
    public function user()
    {
        return $this->belongsTo('App\User','user_id');
    }
    public function expense()
    {
        return $this->belongsTo('App\Models\Expense','expense_id');
    }
    public function sub_expense()
    {
        return $this->belongsTo('App\Models\SubExpense','sub_expense_id');
    }
    public function party_info()
    {
        return $this->belongsTo('App\Models\PartyName', 'party_id');
    }
public function user_expense_details()
    {
        return $this->hasOne('App\Models\UserExpenseDetail','detail_id')->where('type','Expense');
    }
     public function Tags()
    {
        return $this->hasMany('App\Models\ExpenseTag', 'expense_id');
    }
   public function logRemarks()
    {
        return $this->hasMany('App\Models\PaymentRemark', 'rsm_id')->whereIn('type',['expense_issue','expense_update']);
    }
    public function account_to_info()
    {
        return $this->belongsTo('App\Models\Account', 'account_to_id')->with(['bank_info']);
    }
    public function account_from_info()
    {
        return $this->belongsTo('App\Models\Account', 'account_from_id')->with(['bank_info']);
    }
    /**
     * Get the Images of the userExpense.
     */
    public function Images() {
        return $this->hasMany('App\Models\ExpenseImage','user_expense_id','id');
    }
    /***
     * List Expense
     *
     * @return mixed
     */
    public function listExpense(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('amount', 'like', '%' . $request->search_name . '%');
            });
        }
        if ($request->user != '') {
            $data->where('user_id', '=', $request->user);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        $new=0;
        if($id>0){
            $expense=$this->findOrFail($id);
            $new=1;
        }else{
            $expense=new $this();
        }
        $expense->user_id= $request->input('user');
        $expense->expense_id= $request->input('expense_type');
        $expense->date= $request->input('date');
        $expense->amount= $request->input('amount');
        if($expense->save()){
            if($new==1){
                $old_image_id=[];
                $all_product=ExpenseImage::select('id')->where('user_expense_id',$expense->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=ExpenseImage::findOrFail($id);
                        $old_order_product->delete();
                        $file = public_path() .str_replace(url(''),"",$old_order_product['image']) ;
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'expenses/', $filename);
                    if (isset($filename)){
                        $pro_image=new ExpenseImage();
                        $pro_image->image = url('/uploads/expenses/'.$filename);
                        $pro_image->user_expense_id = $expense->id;
                        $pro_image->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
    /***
     * Get User Expense ID using different params
     *
     * @param $userExpenseArray
     * @return int
     */
    public function getUserExpenseId($userExpenseArray)
    {
        $userExpenseRecord =  $this->where('uuid', $userExpenseArray['uuid'])
            ->first();
        return is_null($userExpenseRecord) ? 0 : $userExpenseRecord->id;
    }
    /***
     * Add or Update User Expense Image
     *
     * @param $userExpense
     * @param int $userExpenseId
     *
     * @return mixed
     */
    public function addOrUpdateUserExpenseImage($userExpense, $userExpenseId = 0)
    {

        $userExpenseObj = new $this();
        if ($userExpenseId > 0) {
            $userExpenseObj = $this->where('id', $userExpenseId)
                ->first();

            //User Expense not found
            if (is_null($userExpenseObj)) {
                return $userExpenseObj;
            }
        }
        if(array_key_exists('images', $userExpense)) {
            $images = [];
            foreach($userExpense['images'] as $image) {
                $image = upload_base64_image($image, 'uploads/userExpense/',
                    'userExpenseImage-');
                $userExpenseImage = new ExpenseImage();
                $userExpenseImage->image = 'http://'.$image;
                $images[] = $userExpenseImage;
            }
            if(count($images) > 0) {
                $userExpenseObj->images()->saveMany($images);
            }
        }
        //Remove Image
        if(array_key_exists('remove_images', $userExpense)) {
            $userExpense->images()->whereIn('image', $userExpense['remove_images'])
                ->delete();
            foreach($userExpense['remove_images'] as $image) {
                $fileToUnlink = public_path() . '/uploads/userExpense/' .
                    get_filename_url($image);
                if (file_exists($fileToUnlink)) {
                    unlink($fileToUnlink);
                }
            }
        }
        return $this->getUserExpenseImage($userExpenseObj->id);
    }
    /***
     * Get User Expense Images
     *
     * @param $id
     * @return mixed
     */
    public function getUserExpenseImage($id)
    {
        $userExpense = ExpenseImage::
        where('user_expense_id', $id)
            ->orderBy('id', 'desc')
            ->first();
        return $userExpense;
    }
}
