<?php
/**
 * Created by PhpStorm.
 * User: raza
 * Date: 10/21/17
 * Time: 11:33 PM
 */

namespace App\Models;

use Carbon\Carbon;
use DB;
use Illuminate\Database\Eloquent\Model;

class UserPoint extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $table = 'so_points';
    /***
     * @var array
     */
    protected $guarded = ['id'];
    /***
     * Timestamps does not exists
     *
     * @var bool
     */
    public $timestamps = false;
    protected $appends = ['detail'];
    public function getDetailattribute()
    {
        if ($this->point_type_id == 1) {
            $data = [];
            $data['customer'] = Customer::where('id', $this->detail_id)->with('images')->first();
            $data['display'] = null;
            $data['order'] = null;
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        } else if ($this->point_type_id == 2) {
            $data = [];
            $data['customer'] = null;
            $data['display'] = null;
            $data['order'] = Order::where('id', $this->detail_id)->with('customer')->first();
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        } else if ($this->point_type_id == 5 || $this->point_type_id == 10 ||$this->point_type_id == 11 ) {
            $data = [];
            $data['customer'] =null;
            $data['display'] = null;
            $data['order'] = Order::where('id', $this->detail_id)->with('customer')->first();
            $data['productive_orders'] = Order::where('booked_by', $this->user_id)
                ->Where('created_at', 'like', '%' . $this->date . '%')
                ->with('customer')
                ->get();
            $data['customer_add_target_detail'] = null;
            return $data;
        }else if ($this->point_type_id == 6 ) {
            $data = [];
            $data['customer'] =null;
            $data['display'] = null;
            $data['order'] = Order::where('id', $this->detail_id)->with(['customer'])->first();
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        }else if ( $this->point_type_id == 9) {
            $data = [];
            $data['customer'] =null;
            $data['display'] = null;
            $data['order_product'] = OrderProduct::where('order_id', $this->detail_id)->where('product_id', $this->product_id)->first();
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        }else if ($this->point_type_id == 7) {
            $data = [];
            $data['customer'] =null;
//            $data['customer'] = Customer::where('id', $this->detail_id)->with('images')->first();
            $data['display'] = Display::where('date', $this->date)->where('id', $this->detail_id)->with(['images','customer'=>function($query){return $query->with(['images']);}])->get();
//            $data['display'] = Display::where('date', $this->date)->where('shop_id', $this->detail_id)->with(['images','customer'=>function($query){return $query->with(['images']);}])->get();
            $data['order'] = null;
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        }else if ($this->point_type_id == 8) {
            $data = [];
            $data['customer'] =null;
            $data['display'] = null;
            $data['order'] = null;
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = Customer::where('sale_officer_id',$this->user_id)
                ->where('created_at', 'like', '%' . date("Y-m") . '%')
                ->get();
            return $data;
        } else {
            $data = [];
            $data['customer'] = null;
            $data['display'] = null;
            $data['order'] = null;
            $data['productive_orders'] = null;
            $data['customer_add_target_detail'] = null;
            return $data;
        }

    }
    public function point_type()
    {
        return $this->belongsTo('App\Models\TargetPoint','point_type_id');
    }

    /***
     * Get user points b/w dates
     *
     * @param $userId
     * @param $startDate
     * @param $endDate
     * @param $sendDateWise
     *
     * @return integer $points
     */
    public function getUserPoints($userId, $startDate, $endDate)
    {

            $result =DB::table('so_points as s')
                ->join('target_points as t', 't.id', '=', 's.point_type_id')
                ->select('s.user_id','s.date',  DB::raw("SUM(t.points) as points"))
                ->where('s.user_id',$userId)
                ->whereBetween('s.date', [$startDate,$endDate])
                ->groupBy('s.date')
                ->get();

        return !is_null($result) ? $result : 0;
    }
    public function UserPointsDetails($userId, $Date,$endDate)
    {
        $result=TargetPoint::with(['user_points'=>function($query)use($userId,$Date,$endDate){return $query->where('user_id',$userId)->whereBetween('date', [$Date, $endDate])->get();}])->get();
        return !is_null($result) ? $result : 0;
    }
    public function UserPointsDetail($userId, $Date,$type_id)
    {
        $result =UserPoint::where('user_id',$userId)->where('date',$Date)->where('point_type_id',$type_id)
                ->with(['point_type'=>function($query){return $query->select('id','type','points');}])
                ->get();
        return $result;
    }
    public function UserDozenPointsDetail($userId, $Date,$type_id)
    {
        $result =UserPoint::where('user_id',$userId)->where('date',$Date)->where('point_type_id',$type_id)->groupBy(['detail_id','product_id'])
                ->with(['point_type'=>function($query){return $query->select('id','type','points');}])
                ->get();
//        $result =UserPoint::where('user_id',$userId)->where('date',$Date)->where('point_type_id',$type_id)
//                ->with(['point_type'=>function($query){return $query->select('id','type','points');}])
//                ->groupBy('detail_id')
//                ->get();
        return $result;
    }

    public function UserPointsTenure($userId, $Date,$endDate,$type_id)
    {
        $result=UserPoint::where('user_id',$userId)->whereBetween('date', [$Date, $endDate])->where('point_type_id',  $type_id)->groupBy('date')->selectRaw('count(*) as total, point_type_id,date')->with('point_type')->get();
        return !is_null($result) ? $result : 0;
    }
    public function UserPoints($userId, $startDate, $endDate, $sendDateWise)
    {
        $query = $this->where('user_id', $userId);
        $result = null;
        if(is_null($sendDateWise)) {
            if (!is_null($startDate) && !is_null($endDate)) {
                $query->whereBetween(DB::raw('DATE(date)'), [$startDate, $endDate]);
            } else if (!is_null($startDate)) {
                $query->where(DB::raw('DATE(date)'), '>=', $startDate);
            } else if (!is_null($endDate)) {
                $query->where(DB::raw('DATE(date)'), '<=', $endDate);
            }
            $result = $query->sum('points');
        } else {
            $result = DB::select("SELECT  date, (@totalPoints :=  points  + @totalPoints) AS points
                                    FROM user_points, (SELECT @totalPoints:=0) c WHERE user_points.user_id = $userId;");
        }
        return !is_null($result) ? $result : 0;
    }
    /***
     * @param $userId
     * @param $type
     */
    public function insertUserPoints($userId, $type_id,$detail_id)
    {
        if($type_id==6){
                $this->insert(['user_id' => $userId, 'point_type_id' => $type_id, 'date' => date("Y-m-d"), 'detail_id' => $detail_id]);
        }else if($type_id==5){
            $record=$this->where('user_id',$userId) ->where('date', date("Y-m-d"))->where('point_type_id',5)->first();
            if(empty($record)){
                $this->insert(['user_id' => $userId, 'point_type_id' => $type_id, 'date' => date("Y-m-d"), 'detail_id' => $detail_id]);
            }
        }else if($type_id==3 || $type_id==4|| $type_id==8){
            $record=$this->where('user_id',$userId)
                ->where('date', 'like', '%' . date("Y-m") . '%')
                ->where('point_type_id',$type_id)
                ->first();
            if(empty($record)){
                $this->insert(['user_id' => $userId, 'point_type_id' => $type_id, 'date' => date("Y-m-d"), 'detail_id' => $detail_id]);
            }
        }else{
            $this->insert(['user_id' => $userId, 'point_type_id' => $type_id, 'date' => date("Y-m-d"), 'detail_id' => $detail_id]);
        }
    }
    public function insertdozensalesPoints($userId, $type_id,$detail_id,$product_id)
    {

            $this->insert(['user_id' => $userId, 'point_type_id' => $type_id, 'date' => date("Y-m-d"), 'detail_id' => $detail_id,'product_id' => $product_id]);

    }
    /***
     * Get Expense Types
     */
  public function expenseTypes($request)
    {
         $Role = User::find($request->user_id)->roles->first()->name;
                       $note = ["Credit Note","Debit Note"];
                    if( $Role == 'personal_account' ){

                        $user_id = [9,$request->user_id];
                     
                         $types = Expense::wherein('user_id',$user_id)->whereNotIn('name',$note)->where('country_id',$request->country_id)->orderBy('name')->get();
                       return $types;

                    }if( $Role == 'super_admin' ){
                         $user=User::whereHas('roles', function($query) {
                                    $query->where('name', '=', 'personal_account');
                              })->first();
                        $user_id = [$request->user_id,$user->id];
                         $types = Expense::wherein('user_id',$user_id)->whereNotIn('name',$note)->where('country_id',$request->country_id)->orderBy('name')->get();
                       return $types;

                    }
        $types = Expense::where('user_id',$request->user_id)->where('country_id',$request->country_id)->orderBy('name')->get();
        return $types;
    }
    
  public function expenseDetail($request,$roles)
    {
      
         if($roles == "rsm"){
            
            $UserExpense =  UserExpense::where('id',$request->expense_id)->first();
            $expense_id = $UserExpense->expense_id;
            if ($expense_id == 82){
        $types = UserExpense::where('id',$request->expense_id)->with(['expense','user_expense_details','party_info','logRemarks','account_from_info','account_to_info','Tags.tag'])->first();
    }
    else{

        $types = UserExpense::where('id',$request->expense_id)->with(['Images','expense','user_expense_details','party_info','logRemarks','account_from_info','account_to_info','Tags.tag'])->first();
    }
    }else{
         $types = UserExpense::where('id',$request->expense_id)->with(['Images','expense','sub_expense','user_expense_details','party_info','logRemarks','account_from_info','account_to_info','Tags.tag'])->first();
    }
        return $types;
    }
}